<?php

class AddTorrentDialog extends ProgressDialogManager
{
    const ID = 'add_torrent_dialog';

    const OPEN_TORRENT_TIMEOUT = 60;
    const OPEN_TORRENT_SWITCH_TIMEOUT = 5;

    private $progress;

    private $tfs = null;

    private $switch_mode = 0;
    private $switch_position = -1;
    private $movie_id = null;
    private $season = null;
    private $torrent_id = null;
    private $torrent_uid = null;
    private $stack = null;
    private $hash = null;
    private $chosen_file_id = null;
    private $chosen_file_pos = null;
    private $chosen_op = null;
    private $counter = 0;
    private $tordata = null;

    public $opened = 0;

    public function __construct()
    {
        $this->progress = MainHelper::$progress;
    }

    ///////////////////////////////////////////////////////////////////////

    private function get_open_timeout()
    {
        if (!$this->switch_mode)
            return self::OPEN_TORRENT_TIMEOUT;

        $cfg = MainHelper::$config;
        return $cfg->switch_long_open();
    }

    public function get_series_result_path()
    {
        return DuneSystem::$properties['tmp_dir_path'] . "/series.json";
    }

    public function get_timer() {
        return ActionFactory::timer(950);
    }

    public function get_defs() {
        $defs = array();
        ControlFactory::add_vgap($defs, -22);
        return $defs;
    }

    public function run($switch_mode, $switch_position,
        $movie_id, $season, $torrent_id, $torrent_uid, $hash, $stack,
        $chosen_file_id, $chosen_file_pos, $chosen_op)
    {
        $this->switch_mode = $switch_mode;
        $this->switch_position = $switch_position;
        $this->movie_id = $movie_id;
        $this->season = $season;
        $this->torrent_id = $torrent_id;
        $this->torrent_uid = $torrent_uid;
        $this->hash = $hash;
        $this->stack = $stack;

        $this->chosen_file_id = $chosen_file_id;
        $this->chosen_file_pos = $chosen_file_pos;
        $this->chosen_op = $chosen_op;

        $this->counter = 0;
        $this->tordata = null;
        $this->opened = 1;

        $this->tfs = new TorrentFiles(
            $this->switch_mode, $this->switch_position,
            $this->movie_id, $this->season,
            $this->torrent_id, $this->torrent_uid,
            $this->hash, $this->stack);

        $st = MainHelper::$state;
        if ($this->switch_mode)
            $st->play_torrent($this->tfs, $this->switch_mode);
        else
            $st->enter_torrent($this->tfs);

        $p = $this->progress;
        $p->set_timer_delay(950);
        if ($this->switch_mode)
            return null;

        $p->set_manager($this);
        $this->progress->set_dialog_params(0, 0);
        $p->set_events(array(), array('apply_series', 'close_and_run'));
        $p->set_labels($this->get_labels());
        $this->update_dialog();
        return $p->open();
    }

    ///////////////////////////////////////////////////////////////////////

    private function update_dialog() {
        $this->progress->set_labels($this->get_labels());
    }

    public function get_labels() {
        $labels = array();
        $labels[] = new ProgressDialogLabel(
            T::t('title_searching_for_sources'), HALIGN_CENTER, "14");
        return $labels;
    }

    public function close_and_run($user_input, $action)
    {
        if ($this->switch_mode) {
            $this->on_close($user_input);
            MainHelper::$switch_dialog->set_manager();
            return $action;
        }

        return $this->progress->close_and_run($user_input, $action);
    }

    public function on_event($ev, $user_input)
    {
        if ($ev == 'apply_series')
            return $this->apply_series($user_input);
        if ($ev == 'close_and_run') {
            $str = HD::get($user_input, 'post_action');
            $post_action = $str ? json_decode($str, true) : null;
            return $this->close_and_run($user_input, $post_action);
        }
        return null;
    }

    public function close_allowed() {
        return true;
    }

    public function on_close($user_input) {
        $this->tfs = null;
        $this->opened = 0;
    }

    public function on_timer($user_input)
    {
        $st = MainHelper::$state;
        if ($st->installing_ts)
            return null;

        $this->counter++;

        $this->tordata = TorrServer::torrserverActionGet($this->hash);
        if ($this->tordata && isset($this->tordata->file_stats))
        {
            $this->tfs->set_files($this->tordata->file_stats);
            $series = $this->tfs->get_series();

            if ($series)
            {
                clearstatcache();
                $result_path = $this->get_series_result_path();
                if (is_file($result_path))
                    unlink($result_path);

                $rel_paths = array();
                foreach ($series as $path => $ser)
                    $rel_paths[] = $path;

                $st = MainHelper::$state;
                $mts = $this->switch_mode ? $st->play_mts : $st->nav_mts;
                $t = $mts->get_torrent_by_id($this->tfs->get_torrent_id());
                $base_path = HD::get($t, 'name', '');
                $base_path = str_replace('/', '-', $base_path);

                hd_print("DEBUG running detect_series");

                $a = $this->progress->custom_action('apply_series');
                return ActionFactory::detect_series(
                    $result_path, $rel_paths, $base_path, $a);
            }

            hd_print("AddTorrent dialog timer: bad_torrent");

            if (!$this->switch_mode)
            {
                $a = ActionFactory::show_error(false,
                    T::g('shell_ext_plugin_item_search_no_files_found'));
                return $this->progress->close_and_run($user_input, $a);
            }

            return $this->close_and_run($user_input,
                $this->progress->custom_action('switch_torrent'));
        }

        $open_timeout = $this->get_open_timeout();
        if ($open_timeout > 0 && $this->counter >= $open_timeout)
        {
            hd_print("AddTorrent dialog timer: timed out");

            if (!$this->switch_mode)
            {
                $a = ActionFactory::show_error(false,
                    T::t('errmsg_sources_not_found'));
                return $this->progress->close_and_run($user_input, $a);
            }

            return $this->close_and_run($user_input,
                $this->progress->custom_action('switch_torrent'));
        }

        return null;
    }

    public function apply_series($user_input)
    {
        clearstatcache();
        $res_path = $this->get_series_result_path();
        $doc = is_file($res_path) ? file_get_contents($res_path) : null;
        $jsd = $doc ? json_decode($doc) : null;
        $jsd_dirs = HD::get($jsd, 'dirs', array());
        $sinfo_by_path = array();
        foreach ($jsd_dirs as $d)
        {
            $info_by_name = HD::get(HD::get($d, 'result'), 'info_by_name');
            if (!$info_by_name)
                continue;

            $base = HD::get($d, 'path', '');
            foreach ($info_by_name as $name => $sinfo) {
                $path = (string) $base == '' ? $name : "$base/$name";
                $sinfo_by_path[$path] = $sinfo;
            }
        }

        $err_action = $this->switch_mode ?
            $this->progress->custom_action('switch_torrent') :
            $this->progress->custom_action('close_and_run');

        $series = &$this->tfs->get_series();
        if (!$series) {
            hd_print("New torrent ($this->switch_mode): series list empty");
            return $err_action;
        }

        foreach ($series as $path => &$ser) {
            $sinfo = HD::get($sinfo_by_path, $path);
            if (!$sinfo)
                continue;

            $e = HD::get($sinfo, 'e');
            if ($e && $e >= 0) {
                $ser['e'] = $e;
                $ser['s'] = HD::get($sinfo, 's', 1);
            }
        }

        return $this->apply_torrent($user_input, $err_action);
    }

    private function apply_torrent($user_input, $err_action)
    {
        $st = MainHelper::$state;
        $series = $this->tfs->get_series();
        if (!$series) // Should not happen.
            return $err_action;

        $pos = $this->switch_mode ?
            $this->tfs->get_switch_position() : -1;
        hd_print("DEBUG apply_torrent sw $this->switch_mode pos $pos");

        $enter_file_action = null;
        hd_print("DEBUG apply_torrent chosen_op: $this->chosen_op");
        if (!$this->switch_mode && $this->chosen_op == 'play') {
            $pos = $this->chosen_file_pos;
            $enter_file_action =
                MainHelper::$torrent_files_screen->custom_action(
                    'enter', array('chosen_file_pos' => $pos));
        }

        if ($this->switch_mode)
        {
            $pb = MainHelper::$playback;
            $mts = MainHelper::$state->play_mts;
            $m = $mts->get_movie();
            $s = $mts->get_season();
            $e = $pb->get_episode();
            $sel_id = $m->series_type ? "E$e" : 'single';

            if ($m->series_type) {
                if (!$this->tfs->find_suitable_episode($s, $e)){
                    hd_print("DEBUG torrent misses S$s E$e, skip");
                    return $err_action;
                }

            } else {
                if (count($series) != 1) {
                    hd_print("DEBUG skip multi-file torrent for single movie");
                    return $err_action;
                }
            }

            $vod_movie = TorrentHelper::get_all_episodes_vod_movie(
                $this->stack, $mts->get_autostart(),
                $this->torrent_id, $this->torrent_uid, $e, $pos, $series);
            
            $pb->set_initial_pos($pos);

            $vod_info = $vod_movie->get_vod_info($sel_id, $pos, 1);

            $a = ActionFactory::vod_play($vod_info);
            return $this->progress->close_and_run($user_input, $a);
        }

        // Single-file torrent for single movie, non-switch mode
        // => start playback.

        if (count($series) == 1 && $this->tfs->get_season() < 0 &&
            $this->chosen_op != 'download')
        {
            // NOTE: play_torrent was called on add_torrent_dialog start in
            // case of switch mode
            $st->play_torrent($this->tfs, $this->switch_mode);

            $mts = MainHelper::$state->play_mts;

            $pb = MainHelper::$playback;
            $pb->starting_new_playback();

            $cfg = MainHelper::$config;
            if ($cfg->autoswitch_enabled()) {
                $sel_id = 'single';
                $vod_movie = TorrentHelper::get_all_episodes_vod_movie(
                    $this->stack, $mts->get_autostart(),
                    $this->torrent_id, $this->torrent_uid, -1, $pos, $series);
            } else {
                $vod_movie = $this->tfs->get_torrent_files_vod_movie(
                    $this->stack, $mts->get_autostart());
            }

            hd_print("DEBUG next torrent: (single) play from pos $pos");

            $vod_info = $vod_movie->get_vod_info(
                '', $pos, $this->switch_mode);
            $a = ActionFactory::vod_play($vod_info);
            return $this->progress->close_and_run($user_input, $a);
        }

        // Multi-file torrent otherwise => open folder to browse torrent
        // files.

        $st->enter_torrent($this->tfs);

        if ($this->chosen_op == 'download')
        {
            $a = MainHelper::$entry_handler->custom_action(
                'download_torrent',
                array('torrent_id' => $this->torrent_id));
            return $this->progress->close_and_run($user_input, $a);
        }
        else
        {
            MainHelper::$dl_state->reload_state(
                $this->tfs->get_group_id());

            $files_murl = TorrentFilesScreen::get_media_url_str(
                $this->movie_id, $this->torrent_id, $this->stack);
            $sel_id = $this->chosen_file_id ?
                TorrentFilesScreen::get_item_media_url(
                    $this->chosen_file_id) : null;
            if (!$sel_id && $st->global_e >= 0 && $st->global_s >= 0)
            {
                foreach ($series as $path => $ser)
                {
                    $s = HD::get($ser, 's', -1);
                    $e = HD::get($ser, 'e', -1);
                    if ($e == $st->global_e && $s == $st->global_s)
                    {
                        hd_print("tfiles D: selecting S$s, E$e, path: $path");
                        $sel_id = TorrentFilesScreen::get_item_media_url(
                            $ser['id']);
                        break;
                    }
                }
            }
            $a = ActionFactory::open_folder_ext($files_murl,
                null, null, $sel_id, $enter_file_action);
            return $this->progress->close_and_run($user_input,
                MainHelper::$entry_handler->update_wh($a, false, false));
        }
    }
}

?>
