<?php

class DlState
{
    const GSTAT_PATH = '/tmp/run/dl_gstat.txt';
    const BASE_DIR_PATH = '/flashdata/dl_cache';
    const BASE_TMP_DIR_PATH = '/tmp/run/dl_cache';

    public $gstat = array();
    public $gstat_mtime = -2;

    public $state_group_id = null;
    public $f_by_path = null;
    public $state_mtime = -2;

    public function reload_gstat()
    {
        $path = self::GSTAT_PATH;
        clearstatcache(true, $path);

        if (!is_file($path)) {
            $this->gstat_mtime = -1;
            $changed = (bool) $this->gstat;
            $this->gstat = array();
            return $changed;
        }

        $mtime = filemtime($path);
        if ($mtime === false)
            $mtime = -1;

        if ($mtime == $this->gstat_mtime)
            return false;

        $this->gstat_mtime = $mtime;
        $this->gstat = array();

        $lines = file($path);
        foreach ($lines as $line)
        {
            $line = rtrim($line);
            list ($st, $progress, $id) = explode(' ', $line, 3);
            if ($st == 'D') {
                unset($this->gstat[$id]);
            } else {
                $this->gstat[$id] = (object) array(
                    'status' => $st, 'progress' => $progress);
            }
        }

        return true;
    }

    private function get_torrent_state_path($group_id, $is_tmp)
    {
        $dir = $is_tmp ? self::BASE_TMP_DIR_PATH : self::BASE_DIR_PATH;
        return "$dir/$group_id.json";
    }

    public function sync_state($group_id)
    {
        if ($group_id != $this->state_group_id)
            $this->reload_state($group_id);
    }

    public function reload_state($group_id)
    {
        clearstatcache();
        $path = $this->get_torrent_state_path($group_id, true);
        if (!is_file($path))
            $path = $this->get_torrent_state_path($group_id, false);

        if (!is_file($path)) {
            $changed = $group_id != $this->state_group_id ||
                $this->f_by_path;
            $this->state_mtime = -1;
            $this->f_by_path = null;
            $this->state_group_id = $group_id;
            return $changed;
        }

        $mtime = filemtime($path);
        if ($mtime === false)
            $mtime = -1;

        if ($group_id == $this->state_group_id &&
            $mtime == $this->state_mtime)
        {
            return false;
        }

        $this->state_group_id = $group_id;
        $this->state_mtime = $mtime;
        $this->f_by_path = null;
        $state = json_decode(file_get_contents($path));
        if ($state) {
            foreach ($state->files as $f) {
                $this->f_by_path[$f->path] = $f;
            }
        }
        return true;
    }

    ///////////////////////////////////////////////////////////////////////

    public static function add_download_file(&$files,
        $path, $url, $size_mb)
    {
        $files[] = array(
            'path' => $path, 'url' => $url, 'size_mb' => $size_mb);
    }

    public static function add_download($group_id, $group_name, $files)
    {
        $id = md5($group_id);
        $id = (string) microtime(true) . "." . $id;
        $input_path = "/tmp/run/dl_input/$id";
        hd_print("Download: input_path $input_path");

        $in = array(
            'cmd_type' => 'add',
            'id' => $group_id,
            'name' => $group_name,
            'files' => $files);

        file_put_contents($input_path, json_encode($in));
    }

    public static function remove_download($group_id, $file_paths=null)
    {
        $id = md5($group_id);
        $id = (string) microtime(true) . "." . $id;
        $input_path = "/tmp/run/dl_input/$id";

        $in = array(
            'cmd_type' => 'remove',
            'group_id' => $group_id);
        if ($file_paths)
            $in['file_paths'] = $file_paths;

        file_put_contents($input_path, json_encode($in));
    }
}

?>
