<?php

class LangHelper
{
    // EEE
    const DEBUG_IGNORE = 1;
    const DEBUG_SUITABILITY = 1;

    public static $lang = 'ru';
    public static $avoid_cyr = false;

    public static function set_lang($lang)
    {
        $lang = strtolower($lang);
        self::$lang = $lang == 'ru' ? 'ru' : 'en';
        self::$avoid_cyr =
            $lang != 'ru' && $lang != 'uk' && $lang != 'kk';
    }

    private static function get_suitability($t)
    {
        if (!preg_match('/[\p{Cyrillic}]/u', $t->name))
            return 99;

        $lc = mb_strtolower($t->name, 'UTF-8');

        $w = 0;

        if (preg_match('|\+\s*original|', $lc))
            $w += 4;

        if (preg_match('|оригинальный видеоряд|', $lc))
            $w += 2;

        if (preg_match('|\beng\b|', $lc))
            $w += 1;

        return $w;
    }

    public static function cmp_by_suitability($t1, $t2)
    {
        $s1 = $t1->suitability;
        $s2 = $t2->suitability;
        if ($s1 != $s2)
            return $s2 - $s1;

        return $t1->orig_index - $t2->orig_index;
    }

    public static function filter_torrent_list(&$torrents, &$good_ids)
    {
        if (!$torrents)
            return false;
        if (!self::$avoid_cyr)
            return false;

        $filtered = array();
        foreach ($torrents as $k => $t)
        {
            $n = self::get_suitability($t);
            if (!$n)
            {
                if (self::DEBUG_IGNORE)
                    hd_print("D: ignoring cyrillic caption: $t->name");
                unset($good_ids[$t->id]);
                continue;
            }

            if (self::DEBUG_SUITABILITY)
                hd_print("D: suitability $n for $t->name");
            $t->suitability = $n;
            $t->orig_index = $k;

            $filtered[] = $t;
        }

        usort($filtered, 'LangHelper::cmp_by_suitability');

        $total = count($torrents);
        $num_cyr = $total - count($filtered);
        $perc = sprintf("%.2f", $num_cyr * 100.0 / $total);
        hd_print("Cyr torrents ignored: $num_cyr of total $total ($perc%)");

        $torrents = $filtered;
        return true;
    }
}

?>
