<?php
///////////////////////////////////////////////////////////////////////////

class ActionFactory
{
    public static function open_folder_ext($media_url = null, $caption = null,
        $id = null, $sel_id=null, $post_action = null)
    {
        return array
        (
            GuiAction::handler_string_id => PLUGIN_OPEN_FOLDER_ACTION_ID,
            GuiAction::data =>
                array
                (
                    PluginOpenFolderActionData::media_url => $media_url,
                    PluginOpenFolderActionData::caption => $caption,
                    PluginOpenFolderActionData::id => $id,
                    PluginOpenFolderActionData::sel_id => $sel_id,
                    PluginOpenFolderActionData::post_action => $post_action,
                ),
        );
    }

    public static function open_folder($media_url = null, $caption = null,
        $id = null, $post_action = null)
    {
        return self::open_folder_ext($media_url,
            $caption, $id, null, $post_action);
    }

    public static function vod_play($vod_info = null)
    {
        return array
        (
            GuiAction::handler_string_id => PLUGIN_VOD_PLAY_ACTION_ID,
            GuiAction::data =>
                array
                (
                    PluginVodPlayActionData::vod_info => $vod_info,
                ),
        );
    }

    public static function show_error($fatal, $title,
        $msg_lines=null, $stop_playback=false, $retry_delay_ms=0)
    {
        if ($fatal)
            hd_print("Plugin fatal error.");

        return array
        (
            GuiAction::handler_string_id => PLUGIN_SHOW_ERROR_ACTION_ID,
            GuiAction::caption => null,
            GuiAction::data =>
                array
                (
                    PluginShowErrorActionData::fatal => $fatal,
                    PluginShowErrorActionData::title => $title,
                    PluginShowErrorActionData::msg_lines => $msg_lines,
                    PluginShowErrorActionData::stop_playback => $stop_playback,
                    PluginShowErrorActionData::retry_delay_ms => $retry_delay_ms,
                ),
            GuiAction::params => null,
        );
    }

    public static function show_dialog($title, $defs,
        $close_by_return = false, $preferred_width = 0,
        $attrs = array())
    {
        $initial_sel_ndx = isset($attrs['initial_sel_ndx']) ?
            $attrs['initial_sel_ndx'] : -1;
        $actions = isset($attrs['actions']) ? $attrs['actions'] : null;
        $timer = isset($attrs['timer']) ? $attrs['timer'] : null;
        $min_item_title_width = isset($attrs['min_item_title_width']) ?
            $attrs['min_item_title_width'] : 0;
        $params = isset($attrs['params']) ? $attrs['params'] : null;

        return array
        (
            GuiAction::handler_string_id => SHOW_DIALOG_ACTION_ID,
            GuiAction::caption => null,
            GuiAction::data =>
                array
                (
                    ShowDialogActionData::title => $title,
                    ShowDialogActionData::defs => $defs,
                    ShowDialogActionData::close_by_return => $close_by_return,
                    ShowDialogActionData::preferred_width => $preferred_width,
                    ShowDialogActionData::min_item_title_width => $min_item_title_width,
                    ShowDialogActionData::initial_sel_ndx => $initial_sel_ndx,
                    ShowDialogActionData::actions => $actions,
                    ShowDialogActionData::timer => $timer,
                    ShowDialogActionData::params => $params,
                ),
            GuiAction::params => null,
        );
    }

    public static function close_dialog_and_run($post_action)
    {
        return array
        (
            GuiAction::handler_string_id => CLOSE_DIALOG_AND_RUN_ACTION_ID,
            GuiAction::caption => null,
            GuiAction::data =>
                array
                (
                    CloseDialogAndRunActionData::post_action => $post_action,
                ),
            GuiAction::params => null,
        );
    }

    public static function close_dialog()
    {
        return self::close_dialog_and_run(null);
    }

    public static function close_and_run($post_action=null)
    {
        return array
        (
            GuiAction::handler_string_id => CLOSE_AND_RUN_ACTION_ID,
            GuiAction::caption => null,
            GuiAction::data =>
                array
                (
                    CloseAndRunActionData::post_action => $post_action,
                ),
            GuiAction::params => null,
        );
    }

    public static function show_title_dialog($title, $post_action = null)
    {
        $defs = array();

//        ControlFactory::add_vgap($defs, 50);

        ControlFactory::add_custom_close_dialog_and_apply_button($defs,
            'ok', 'OK', 300, $post_action);

        return self::show_dialog($title, $defs);
    }

    public static function show_message_dialog($title, $messages, $post_action = null)
    {
        $defs = array();

        if ($messages && !is_array($messages))
            $messages = array($messages);

        foreach ($messages as $msg)
        {
            ControlFactory::add_label($defs, null, $msg);
            ControlFactory::add_vgap($defs, -15);
        }

        ControlFactory::add_vgap($defs, 30);
        ControlFactory::add_custom_close_dialog_and_apply_button($defs,
            'ok', 'OK', 300, $post_action);

        return self::show_dialog($title, $defs);
    }

    public static function show_yes_no_dialog($title,
        $messages, $yes_action, $close_by_return = false, $yes_by_default=true)
    {
        $defs = array();

        if ($messages)
        {
            foreach ($messages as $msg)
                ControlFactory::add_label($defs, null, $msg);
            ControlFactory::add_vgap($defs, 20);
        }

        ControlFactory::add_custom_button($defs,
            self::close_dialog_and_run($yes_action),
            'yes', null, T::key_global('yes_no_choice_yes'), 300);

        ControlFactory::add_close_dialog_button($defs,
            T::key_global('yes_no_choice_no'), 300);

        return self::show_dialog($title, $defs, $close_by_return, 0,
            array('initial_sel_ndx' => $yes_by_default ? 0 : 1));
    }

    public static function show_waiting_dialog($title, $messages, $post_action)
    {
        $timer = ActionFactory::timer(0);
        $actions = array(GUI_EVENT_TIMER => $post_action);
        $defs = array();
        if ($messages)
        {
            $params = array('color' => 14);
            foreach ($messages as $msg)
                ControlFactory::add_label($defs, null, $msg, $params);
        }
        else
            ControlFactory::add_vgap($defs, -25);
        return ActionFactory::show_dialog($title, $defs, false, 0,
            array('actions' => $actions, 'timer' => $timer));
    }

    public static function status($status)
    {
        return array
        (
            GuiAction::handler_string_id => STATUS_ACTION_ID,
            GuiAction::caption => null,
            GuiAction::data =>
                array
                (
                    StatusActionData::status => $status,
                ),
            GuiAction::params => null,
        );
    }

    public static function invalidate_folders($media_urls,
        $post_action = null, $all_except = false)
    {
        return array
        (
            GuiAction::handler_string_id => PLUGIN_INVALIDATE_FOLDERS_ACTION_ID,
            GuiAction::data =>
                array
                (
                    PluginInvalidateFoldersActionData::media_urls => $media_urls,
                    PluginInvalidateFoldersActionData::post_action => $post_action,
                    PluginInvalidateFoldersActionData::all_except => $all_except,
                ),
        );
    }

    public static function invalidate_all_folders($post_action = null,
        $except_media_urls = null)
    {
        return self::invalidate_folders($except_media_urls, $post_action, true);
    }

    public static function show_popup_menu($menu_items, $sel_ndx = 0)
    {
        return array
        (
            GuiAction::handler_string_id => SHOW_POPUP_MENU_ACTION_ID,
            GuiAction::data =>
                array
                (
                    ShowPopupMenuActionData::menu_items => $menu_items,
                    ShowPopupMenuActionData::selected_menu_item_index => $sel_ndx,
                ),
        );
    }

    public static function get_menu_item($caption, $action)
    {
        return array(
            GuiMenuItemDef::caption => $caption,
            GuiMenuItemDef::action => $action);
    }

    public static function get_osd_menu_item($id, $caption, $marked)
    {
        $arr = array(GuiMenuItemDef::caption => $caption);
        if (isset($id))
            $arr[GuiMenuItemDef::id] = $id;
        if ($marked)
            $arr[GuiMenuItemDef::marked] = $marked;
        return $arr;
    }

    public static function update_regular_folder($range,
        $need_refresh = false, $sel_ndx = -1)
    {
        return array
        (
            GuiAction::handler_string_id => PLUGIN_UPDATE_FOLDER_ACTION_ID,
            GuiAction::data =>
                array
                (
                    PluginUpdateFolderActionData::range => $range,
                    PluginUpdateFolderActionData::need_refresh => $need_refresh,
                    PluginUpdateFolderActionData::sel_ndx => intval($sel_ndx),
                ),
        );
    }

    public static function reset_controls($defs, $post_action = null, $initial_sel_ndx = -1)
    {
        return array
        (
             GuiAction::handler_string_id => RESET_CONTROLS_ACTION_ID,
             GuiAction::data =>
             array
             (
                ResetControlsActionData::defs => $defs,
                ResetControlsActionData::initial_sel_ndx => $initial_sel_ndx,
                ResetControlsActionData::post_action => $post_action,
             ),
        );
    }

    public static function launch_media_url($url)
    {
        return array
        (
             GuiAction::handler_string_id => LAUNCH_MEDIA_URL_ACTION_ID,
             GuiAction::data =>
             array
             (
                LaunchMediaUrlActionData::url => $url,
             ),
        );
    }

    public static function stop_playback($wait_for_completion=false, $post_action=null)
    {
        return array
        (
             GuiAction::handler_string_id => STOP_PLAYBACK_ACTION_ID,
             GuiAction::data =>
             array
             (
                StopPlaybackActionData::wait_for_completion => $wait_for_completion,
                StopPlaybackActionData::post_action => $post_action,
             ),
        );
    }

    public static function handle_user_input($params,
        $timeout_ms=null, $show_dialog_delay_ms=null, $plugin_name=null)
    {
        $data = null;
        if (isset($timeout_ms) || isset($show_dialog_delay_ms))
        {
            $data = array();
            if (isset($timeout_ms))
            {
                $data[PluginHandleUserInputActionData::operation_timeout] =
                    intval($timeout_ms);
            }
            if (isset($show_dialog_delay_ms))
            {
                $data[PluginHandleUserInputActionData::show_dialog_delay] =
                    intval($show_dialog_delay_ms);
            }
        }

        $arr = array
        (
            GuiAction::handler_string_id => PLUGIN_HANDLE_USER_INPUT_ACTION_ID,
            GuiAction::caption => null,
            GuiAction::data => $data,
            GuiAction::params => $params,
        );
        if ($plugin_name)
            $arr[GuiAction::plugin_name] = $plugin_name;
        return $arr;
    }

    public static function change_behaviour($actions, $timer=null, $post_action=null)
    {
        return array
        (
             GuiAction::handler_string_id => CHANGE_BEHAVIOUR_ACTION_ID,
             GuiAction::data =>
             array
             (
                ChangeBehaviourActionData::actions => $actions,
                ChangeBehaviourActionData::timer => $timer,
                ChangeBehaviourActionData::post_action => $post_action,
             ),
        );
    }

    public static function add_menu_items(
        $menu_id, $menu_items, $sel_ndx=-1, $menu_params=null, $post_action=null)
    {
        return array
        (
            GuiAction::handler_string_id => ADD_MENU_ITEMS_ACTION_ID,
            GuiAction::data =>
                array
                (
                    AddMenuItemsActionData::menu_id => $menu_id,
                    AddMenuItemsActionData::menu_params => $menu_params,
                    AddMenuItemsActionData::menu_items => $menu_items,
                    AddMenuItemsActionData::sel_ndx => $sel_ndx,
                    AddMenuItemsActionData::post_action => $post_action,
                ),
        );
    }

    public static function add_path_element(&$elements,
        $media_url, $caption, $id)
    {
        $elements[] = array(
            PluginPathElement::media_url => $media_url,
            PluginPathElement::caption => $caption,
            PluginPathElement::id => $id);
    }

    public static function replace_path($erase_count, $elements = null, $sel_id=null,
        $post_action = null)
    {
        return array
        (
            GuiAction::handler_string_id => PLUGIN_REPLACE_PATH_ACTION_ID,
            GuiAction::data =>
                array
                (
                    PluginReplacePathActionData::erase_count => $erase_count,
                    PluginReplacePathActionData::elements => $elements,
                    PluginReplacePathActionData::sel_id => $sel_id,
                    PluginReplacePathActionData::post_action => $post_action,
                ),
        );
    }

    public static function plugin_system($run_string)
    {
        return array
        (
             GuiAction::handler_string_id => PLUGIN_SYSTEM_ACTION_ID,
             GuiAction::data =>
             array
             (
                PluginSystemActionData::run_string => $run_string,
             ),
        );
    }

    public static function wget($url, $post_data=null, $curl_opts=null,
        $id=null, $target_path=null,
        $result_key_prefix=null, $post_action=null)
    {
        return array
        (
             GuiAction::handler_string_id => WGET_ACTION_ID,
             GuiAction::data =>
             array
             (
                WgetActionData::id => $id,
                WgetActionData::url => $url,
                WgetActionData::post_data => $post_data,
                WgetActionData::curl_opts => $curl_opts,
                WgetActionData::target_path => $target_path,
                WgetActionData::result_key_prefix => $result_key_prefix,
                WgetActionData::post_action => $post_action,
             ),
        );
    }

    public static function detect_series($result_path,
        $rel_paths, $base_path=null, $post_action=null)
    {
        return array
        (
             GuiAction::handler_string_id => DETECT_SERIES_ACTION_ID,
             GuiAction::data =>
             array
             (
                DetectSeriesActionData::result_path => $result_path,
                DetectSeriesActionData::rel_paths => $rel_paths,
                DetectSeriesActionData::base_path => $base_path,
                DetectSeriesActionData::post_action => $post_action,
             ),
        );
    }

    public static function composite($actions)
    {
        $arr = array();
        foreach ($actions as $a) {
            if ($a) {
                hd_print("composite D: +".$a['handler_string_id']);
                $arr[] = $a;
            }
        }

        if (!$arr)
            return null;
        if (count($arr) == 1)
            return $arr[0];

        return array
        (
             GuiAction::handler_string_id => COMPOSITE_ACTION_ID,
             GuiAction::data =>
             array
             (
                CompositeActionData::actions => $arr,
             ),
        );
    }

    public static function show_ok_cancel_dialog($title,
        $messages, $ok_action)
    {
        $defs = array();

        foreach ($messages as $message)
            ControlFactory::add_label($defs, null, $message);

        if (count($messages) > 0)
            ControlFactory::add_vgap($defs, 20);

        ControlFactory::add_custom_button($defs,
            self::close_dialog_and_run($ok_action),
            'ok', null, T::key_global('button_ok'), 300);

        ControlFactory::add_close_dialog_button($defs,
            T::key_global('button_cancel'), 300);

        return self::show_dialog($title, $defs);
    }


    public static function wait_ext_app_start(
        $delay, $package, $error_action=null)
    {
        return array
        (
             GuiAction::handler_string_id => WAIT_EXT_APP_START_ACTION_ID,
             GuiAction::data =>
             array
             (
                WaitExtAppStartActionData::delay => $delay,
                WaitExtAppStartActionData::package => $package,
                WaitExtAppStartActionData::error_action => $error_action,
             ),
        );
    }

    public static function launch_ext_app(
        $cmd, $ensure_playback_stopped, $delay, $package,
        $error_action=null)
    {
        return array
        (
             GuiAction::handler_string_id => LAUNCH_EXT_APP_ACTION_ID,
             GuiAction::data =>
             array
             (
                LaunchExtAppActionData::cmd => $cmd,
                LaunchExtAppActionData::ensure_playback_stopped => $ensure_playback_stopped,
                LaunchExtAppActionData::delay => $delay,
                LaunchExtAppActionData::package => $package,
                LaunchExtAppActionData::error_action => $error_action,
             ),
        );
    }

    public static function download_and_install_apk(
        $caption, $package, $urls, $indirect=false, $post_action=null)
    {
        return array
        (
             GuiAction::handler_string_id => DOWNLOAD_AND_INSTALL_APK_ACTION_ID,
             GuiAction::data =>
             array
             (
                DownloadAndInstallApkActionData::caption => $caption,
                DownloadAndInstallApkActionData::package => $package,
                DownloadAndInstallApkActionData::urls => $urls,
                DownloadAndInstallApkActionData::indirect => $indirect,
                DownloadAndInstallApkActionData::post_action => $post_action,
             ),
        );
    }

    public static function add_to_favorites(
        $media_url, $post_action=null)
    {
        return array
        (
             GuiAction::handler_string_id => ADD_TO_FAVORITES_ACTION_ID,
             GuiAction::data =>
             array
             (
                AddToFavoritesActionData::media_url => $media_url,
                AddToFavoritesActionData::post_action => $post_action,
             ),
        );
    }

    public static function set_restricted(
        $id, $restricted, $post_action=null)
    {
        return array
        (
             GuiAction::handler_string_id => SET_RESTRICTED_ACTION_ID,
             GuiAction::data =>
             array
             (
                SetRestrictedActionData::id => $id,
                SetRestrictedActionData::restricted => $restricted,
                SetRestrictedActionData::post_action => $post_action,
             ),
        );
    }

    public static function check_restricted(
        $id, $post_action)
    {
        return array
        (
             GuiAction::handler_string_id => CHECK_RESTRICTED_ACTION_ID,
             GuiAction::data =>
             array
             (
                CheckRestrictedActionData::id => $id,
                CheckRestrictedActionData::post_action => $post_action,
             ),
        );
    }

    public static function restart($reboot)
    {
        return array
        (
             GuiAction::handler_string_id => RESTART_ACTION_ID,
             GuiAction::data =>
             array
             (
                RestartActionData::reboot => $reboot,
             ),
        );
    }

    public static function update_tv_info($post_action=null)
    {
        return array
        (
             GuiAction::handler_string_id => UPDATE_TV_INFO_ACTION_ID,
             GuiAction::data =>
             array
             (
                UpdateTvInfoActionData::post_action => $post_action,
             ),
        );
    }

    public static function bt_command($name, $params=null, $callback_action=null)
    {
        return array
        (
             GuiAction::handler_string_id => BT_COMMAND_ACTION_ID,
             GuiAction::data =>
             array
             (
                BtCommandActionData::name => $name,
                BtCommandActionData::params => $params,
                BtCommandActionData::callback_action => $callback_action,
             ),
        );
    }

    public static function timer($delay_ms, $reset_on_keypress=false, $early_start=false)
    {
        $arr = array(GuiTimerDef::delay_ms => $delay_ms);
        if ($reset_on_keypress)
            $arr[GuiTimerDef::reset_on_keypress] = true;
        if ($early_start)
            $arr[GuiTimerDef::early_start] = true;
        return $arr;
    }

    public static function file_movie_info(
        $movie_id, $title, $icon_url, $s=-1, $e=-1)
    {
        $arr = array(
            FileMovieInfo::movie_id => $movie_id,
            FileMovieInfo::title => $title,
            FileMovieInfo::icon_url => $icon_url);
        if ($e >= 0) {
            if ($s >= 0)
                $arr[FileMovieInfo::s] = $s;
            $arr[FileMovieInfo::e] = $e;
        }
        return $arr;
    }

    public static function decode_action(
        $user_input, $key, $res_on_error=null)
    {
        $str = HD::get($user_input, $key);
        return $str ? json_decode($str, true) : $res_on_error;
    }

    public static function decode_result_action(
        $user_input, $res_on_error=null)
    {
        return self::decode_action(
            $user_input, 'result_action', $res_on_error);
    }

    public static function action_from_user_input($user_input,
        $add_params=null, $rem_keys=null)
    {
        $arr = array();
        foreach ($user_input as $key => $val)
            $arr[$key] = $val;
        if ($add_params) {
            foreach ($add_params as $key => $val)
                $arr[$key] = $val;
        }
        if ($rem_keys) {
            foreach ($rem_keys as $key)
                unset($arr[$key]);
        }
        $a = self::handle_user_input($arr);
        $a[GuiAction::plugin_name] =
            DuneSystem::$properties['plugin_name'];
        return $a;
    }
}

///////////////////////////////////////////////////////////////////////////
?>
