<?php

class ShortMovie
{
    public $id;
    public $name;
    public $poster_url;
    public $year;
    public $detailed_poster_url;

    public function __construct($id, $name, $poster_url, $year=0)
    {
        if (is_null($id))
            throw new Exception("ShortMovie::id is null");

        $this->id = strval($id);
        $this->name = strval($name);
        $this->poster_url = strval($poster_url);
        $this->year = $year;
        $this->detailed_poster_url = null;
    }
}

class ShortMovieRange
{
    public $from_ndx;
    public $total;
    public $short_movies;

    public function __construct($from_ndx, $total, $short_movies = null)
    {
        $this->from_ndx = intval($from_ndx);
        $this->total = intval($total);
        $this->short_movies = $short_movies === null ?
            array() : $short_movies;
    }
}

class MovieSeries
{
    public $id;

    public function __construct($id)
    {
        if (is_null($id))
            throw new Exception("MovieSeries::id is null");

        $this->id = strval($id);
    }

    public $name = '';
    public $playback_url = '';
    public $playback_url_is_stream_url = true;
    public $recent_uid = null;
    public $mdb_info = null;
}

class Movie
{
    public $id;
    public $name = '';
    public $name_original = '';
    public $sub_name = '';
    public $description = '';
    public $poster_url = '';
    public $length_min = -1;
    public $year = 0;
    public $directors_str = '';
    public $scenarios_str = '';
    public $actors_str = '';
    public $genres_str = '';
    public $rate_imdb = '';
    public $rate_kinopoisk = '';
    public $rate_mpaa = '';
    public $country = '';
    public $budget = '';
    public $details = null;
    public $rate_details = null;

    public $series_list = null;

    public $ext;

    public $async_data;

    public $extra_menu_items = null;
    public $enter_menu_action = null;

    public $recent_plugin = null;
    public $recent_action_params = null;
    public $recent_group_by = null;
    public $recent_tags = null;

    public $actions = null;
    public $timer = null;

    public $initial_position = -1;

    public function __construct($id)
    {
        if (is_null($id))
            throw new Exception("Movie::id is null");

        $this->id = strval($id);

        $this->ext = (object) array();

        $this->async_data = (object) array();
    }

    public function is_loaded($ext)
    {
        return $ext ?
            isset($this->async_data->ext_completed) :
            isset($this->async_data->base_completed);
    }

    public function set_loaded($ext)
    {
        $this->async_data->base_completed = true;
        if ($ext)
            $this->async_data->ext_completed = true;
    }

    public static function to_string($v)
    {
        return $v === null ? '' : strval($v);
    }

    public static function to_int($v, $default_value)
    {
        $v = strval($v);
        if (!is_numeric($v))
            return $default_value;
        $v = intval($v);
        return $v <= 0 ? $default_value : $v;
    }

    public function set_data(
        $name,
        $name_original,
        $description,
        $poster_url,
        $length_min,
        $year,
        $directors_str,
        $scenarios_str,
        $actors_str,
        $genres_str,
        $rate_imdb,
        $rate_kinopoisk,
        $rate_mpaa,
        $country,
        $budget)
    {
        $this->name = self::to_string($name);
        $this->name_original = self::to_string($name_original);
        $this->description = self::to_string($description);
        $this->poster_url = self::to_string($poster_url);
        $this->length_min = self::to_int($length_min, -1);
        $this->year = self::to_int($year, -1);
        $this->directors_str = self::to_string($directors_str);
        $this->scenarios_str = self::to_string($scenarios_str);
        $this->actors_str = self::to_string($actors_str);
        $this->genres_str = self::to_string($genres_str);
        $this->rate_imdb = self::to_string($rate_imdb);
        $this->rate_kinopoisk = self::to_string($rate_kinopoisk);
        $this->rate_mpaa = self::to_string($rate_mpaa);
        $this->country = self::to_string($country);
        $this->budget = self::to_string($budget);

        $this->series_list = array();
    }

    public function set_details($details)
    {
        $this->details = $details;
    }

    public function add_series_data($id, $name,
        $playback_url, $playback_url_is_stream_url,
        $recent_uid=null, $mdb_info=null)
    {
        $series = new MovieSeries($id);

        $series->name = self::to_string($name);
        $series->playback_url = self::to_string($playback_url);
        $series->playback_url_is_stream_url =
            $playback_url_is_stream_url === true;
        $series->recent_uid = $recent_uid;
        $series->mdb_info = $mdb_info;

        $this->series_list[] = $series;
    }

    public function set_behaviour($actions, $timer=null)
    {
        $this->actions = $actions;
        $this->timer = $timer;
    }

    public function set_extra_menu($extra_menu_items, $enter_menu_action)
    {
        $this->extra_menu_items = $extra_menu_items;
        $this->enter_menu_action = $enter_menu_action;
    }

    public function get_movie_array()
    {
        return array(
            PluginMovie::name => $this->name,
            PluginMovie::name_original => $this->name_original,
            PluginMovie::description => $this->description,
            PluginMovie::poster_url => $this->poster_url,
            PluginMovie::length_min => $this->length_min,
            PluginMovie::year => $this->year,
            PluginMovie::directors_str => $this->directors_str,
            PluginMovie::scenarios_str => $this->scenarios_str,
            PluginMovie::actors_str => $this->actors_str,
            PluginMovie::genres_str => $this->genres_str,
            PluginMovie::rate_imdb => $this->rate_imdb,
            PluginMovie::rate_kinopoisk => $this->rate_kinopoisk,
            PluginMovie::rate_mpaa => $this->rate_mpaa,
            PluginMovie::country => $this->country,
            PluginMovie::budget => $this->budget,
            PluginMovie::details => $this->details,
            PluginMovie::rate_details => $this->rate_details,
        );
    }

    public function get_vod_info($sel_id, $initial_position=-1,
        $skip_dummy_player_on_error=false)
    {
        if (!is_array($this->series_list) ||
            count($this->series_list) == 0)
        {
            throw new Exception('Invalid movie: series list is empty');
        }

        $series_array = array();
        $initial_series_ndx = -1;
        foreach ($this->series_list as $ndx => $series)
        {
            if (!is_null($sel_id) && $series->id === $sel_id)
                $initial_series_ndx = $ndx;

            $arr = array(
                PluginVodSeriesInfo::name => $series->name,
                PluginVodSeriesInfo::playback_url => $series->playback_url,
                PluginVodSeriesInfo::playback_url_is_stream_url =>
                    $series->playback_url_is_stream_url,
            );
            if ($series->recent_uid)
                $arr[PluginVodSeriesInfo::recent_uid] = $series->recent_uid;
            if ($series->mdb_info)
                $arr[PluginVodSeriesInfo::mdb_info] = $series->mdb_info;
            $series_array[] = $arr;
        }

        $arr = array(
            PluginVodInfo::id => $this->id,
            PluginVodInfo::name => $this->name,
            PluginVodInfo::description => $this->description,
            PluginVodInfo::poster_url => $this->poster_url,
            PluginVodInfo::series => $series_array,
            PluginVodInfo::initial_series_ndx => $initial_series_ndx,
            PluginVodInfo::buffering_ms => 3000,
            PluginVodInfo::keep_playing_on_reenter => true,
            PluginVodInfo::skip_dummy_player_on_error =>
                (bool) $skip_dummy_player_on_error,
        );
        if ($this->sub_name)
            $arr[PluginVodInfo::sub_name] = $this->sub_name;

        if ($initial_position < 0)
            $initial_position = $this->initial_position;
        if ($initial_position >= 0) {
            hd_print("DEBUG vod_play final initial_position: $initial_position");
            $arr[PluginVodInfo::initial_position_ms] =
                $initial_position * 1000;
        }
        if ($this->actions)
            $arr[PluginVodInfo::actions] = $this->actions;
        if ($this->timer)
            $arr[PluginVodInfo::timer] = $this->timer;
        if ($this->recent_plugin && $this->recent_action_params)
        {
            $arr[PluginVodInfo::recent_plugin] = $this->recent_plugin;
            $arr[PluginVodInfo::recent_action_params] =
                $this->recent_action_params;
            if (isset($this->recent_group_by))
            {
                $arr[PluginVodInfo::recent_group_by] =
                    $this->recent_group_by;
            }
            if ($this->recent_tags)
                $arr[PluginVodInfo::recent_tags] = $this->recent_tags;
        }
        if ($this->extra_menu_items && $this->enter_menu_action)
        {
            $arr[PluginVodInfo::extra_menu_items] =
                $this->extra_menu_items;
            $arr[PluginVodInfo::enter_menu_action] =
                $this->enter_menu_action;
        }
        return $arr;
    }
}

?>
