<?php
///////////////////////////////////////////////////////////////////////////

require_once 'lib/user_input_handler_registry.php';

class MainEntryHandler implements UserInputHandler
{
    public function __construct()
    {
        UserInputHandlerRegistry::get_instance()->
            register_handler($this);
    }

    ///////////////////////////////////////////////////////////////////////

    public function custom_action($id, $add_params=null)
    {
        return UserInputHandlerRegistry::create_action(
            $this, $id, $add_params, 10000, 11000);
    }

    public function get_handler_id()
    {
        return 'entry';
    }

    public function update_wh($post_action, $with_play_tfs, $force_reload)
    {
        $th = MainHelper::$torrent_history;

        $result_path = $th->get_wh_path();
        $cb = $this->custom_action(
            'post_update_wh',
            array(
                'result_file' => $result_path,
                'post_action' => json_encode($post_action)));
        $cb[GuiAction::plugin_name] =
            DuneSystem::$properties['plugin_name'];

        $st = MainHelper::$state;
        $mts = $st->nav_mts;
        $tfs = $st->nav_tfs;
        $m = $mts->get_movie();
        $series = $tfs ? $tfs->get_series() : array();

        $uids = array();
        if ($tfs) {
            foreach ($series as $ser) {
                $uids[] = $th->make_file_uid($m->id,
                    $tfs->get_torrent_uid(), $ser['id']);
            }
        }

        if ($with_play_tfs)
        {
            $play_mts = $st->play_mts;
            $play_tfs = $st->play_tfs;
            if ($play_mts && $play_tfs && $with_play_tfs &&
                (!$tfs || $play_tfs->get_torrent_id() != $tfs->get_torrent_id()))
            {
                $play_series = $play_tfs->get_series();
                foreach ($play_series as $ser) {
                    $uids[] = $th->make_file_uid($m->id,
                        $play_tfs->get_torrent_uid(), $ser['id']);
                }
            }
        }

        $tags = array();
        foreach ($mts->get_torrents() as $t) {
            $tags[] = $th->make_torrent_tag($t->uid);
        }

        return ActionFactory::handle_user_input(
            array(
                'handler_id' => 'entry',
                'control_id' => 'find_recent',
                'force_reload' => $force_reload,
                'uids' => json_encode($uids),
                'tags' => json_encode($tags),
                'result_action' => json_encode($cb),
                'result_file' => $result_path,
            ), null, null, 'shell_ext');
    }

    public function update_eps_wh($movie_id, $s, $e, $post_action)
    {
        $th = MainHelper::$torrent_history;
        $result_path = $th->get_eps_wh_path();
        $cb = $this->custom_action(
            'post_update_wh',
            array(
                'result_file' => $result_path,
                'eps_mode' => 1,
                'post_action' => json_encode($post_action)));
        $cb[GuiAction::plugin_name] =
            DuneSystem::$properties['plugin_name'];

        $ep = $th->make_ep_tag($movie_id, $s, $e);

        return ActionFactory::handle_user_input(
            array(
                'handler_id' => 'entry',
                'control_id' => 'find_recent',
                'force_reload' => false,
                'eps' => json_encode(array($ep)),
                'result_action' => json_encode($cb),
                'result_file' => $result_path,
            ), null, null, 'shell_ext');
    }

    private function install_ts($user_input, $result_action)
    {
        $info = $this->load_ts_info();
        if (!$info)
        {
            $p = MainHelper::$progress;
            return $p->close_and_run($user_input,
                $this->torrserver_not_started());
        }

        $st = MainHelper::$state;
        $st->installing_ts = 1;

        $ra_str = json_encode($result_action);
        $a = $this->custom_action('apply_ltsb',
            array('result_action' => $ra_str));

        $dirpath = "/flashdata/ts";
        $binpath_l = "$dirpath/ts.loading";

        $wget_a = ActionFactory::wget($info->url, null, 
            array(CURLOPT_TIMEOUT => 600),
            'load_ts_bin', $binpath_l, 'ltsb_', $a);
        return ActionFactory::show_waiting_dialog(
            T::t('title_preparing'), null, $wget_a);
    }

    /**
     * This is main entry point to this plugin, it is called by shell_ext
     * plugin.
     */
    private function play(&$user_input)
    {
        $movie_str = HD::get($user_input, 'movie_str');
        $lang = HD::get($user_input, 'lang');
        $region = HD::get($user_input, 'region');

        if (!$movie_str || !$lang || !$region)
        {
            hd_print("Play: required arguments are missing");
            return null;
        }

        LangHelper::set_lang($lang);

        // Legacy.
        TorLook_Class::setLang($lang);

        MainHelper::$config->reload();

        $sup_id = HD::get($user_input, 'supplier_id', '');
        $manual = 'online_movies_manual' == $sup_id;
        if (!$manual && !HD::is_apk() && !self::is_ts_installed())
        {
            $a = ActionFactory::action_from_user_input($user_input);
            return $this->install_ts($user_input, $a);
        }

        $st = MainHelper::$state;
        $is_playing = 'plugin_vod' == HD::get($user_input, 'play_mode', 'none');
        $autostart = 1 != HD::get($user_input, 'manual', 0);
        if (!$st->enter_movie($movie_str, $lang, $region,
                $is_playing, $autostart))
        {
            hd_print("No torrents found for movie.");
            return ActionFactory::show_error(false,
                T::g('shell_ext_plugin_item_search_no_files_found'));
        }

        $arr = array('control_id' => 'do_play');

        $e = HD::get($user_input, 'episode', -1);
        $s = HD::get($user_input, 'season', -1);
        if ($e >= 0) {
            // Use-case: Movie -> Seasons -> Season[X] -> Episode[Y] ->
            // Online Movies (...)
            $arr['chosen_s'] = $s;
            $arr['chosen_e'] = $e;
            $arr['chosen_se_provided'] = 1;
            if (!$manual)
                $arr['user_choice'] = 'auto_choose';
        }

        $st->global_e = $e;
        $st->global_s = $e > 0 ? ($s < 0 ? 1 : $s) : $s;
        
        $rem = array('movie_str', 'lang', 'region');

        $cb = ActionFactory::action_from_user_input(
            $user_input, $arr, $rem);
        return $this->update_wh($cb, false, false);
    }

    private static function format_play_position($sec)
    {
        $res = '';
		$hour = floor($sec / 3600);
		if ($hour > 0)
            $res .= $hour . ':';
		$sec -= $hour * 3600;
		$min = floor($sec / 60);
		$sec -= $min * 60;
		if ($hour > 0)
            $res .= sprintf("%02d:%02d", $min, $sec);
        else
            $res .= sprintf("%d:%02d", $min, $sec);
        return $res;
    }

    private static function get_resume_caption($witem, $m)
    {
        $percent = $witem->len > 0 ?
            round($witem->pos * 100.0 / $witem->len) : 0;
        $pos_str = self::format_play_position($witem->pos);
        $len_str = self::format_play_position($witem->len);

        $e = HD::get($witem, 'e', -1);
        if ($m->series_type && $e >= 0)
        {
            $s = HD::get($witem, 's', 1);
            $se_str = T::g2(
                'shell_ext_plugin_item_season_episode_short__2', $s, $e);
            return T::g4('resume_playback_button_ext__4',
                $se_str, $pos_str, $len_str, $percent);
        } else {
            return T::g3('resume_playback_button__3',
                $pos_str, $len_str, $percent);
        }
    }

    private function add_smart_button(&$defs, $first,
        $name, $caption, $icon_url, $add_params, $gui_params)
    {
        if (!$first)
            ControlFactory::add_vgap($defs, -10);

        $ICON_W = 48;
        $ICON_H = 48;
        $ICON_RIGHT = 20;
        $BUTTON_W = 1200;
        $text_dy = ($ICON_H - 48) / 2;

        $items = array();
        ControlFactory::add_smart_gap($items, $ICON_RIGHT);
        ControlFactory::add_smart_icon($items, $icon_url, $ICON_W, $ICON_H);
        ControlFactory::add_smart_gap($items, $ICON_RIGHT);
        ControlFactory::add_smart_text($items, $caption, $text_dy);

        $caption = json_encode(array('items' => $items));
        $action = $name ? $this->custom_action($name, $add_params) : null;
        $action = ActionFactory::close_dialog_and_run($action);
        ControlFactory::add_custom_button($defs, $action,
            $name, null, $caption, $BUTTON_W, $gui_params);
    }

    private function make_user_choice_on_enter(&$user_input,
        $choices, $witem, $m)
    {
        $button_gui_params = array(
            'button_caption_centered' => 'no',
            'button_style' => 'smart_label',
            'button_height' => 54,
        );

        $add_params = array();
        foreach (array('movie_str', 'lang', 'region', 'supplier_info',
                'chosen_s', 'chosen_e', 'eps_wh_updated',
                'tv_info_updated',
                'chosen_se_provided', 'ignore_wh') as $k)
        {
            $add_params[$k] = HD::get($user_input, $k);
        }

        $icon = "plugin_file://%shell_ext%/icons/movies_small.png";
        foreach ($choices as $ndx => $choice)
        {
            $id = "user_choice_$choice";
            if ($choice == 'resume')
                $caption = self::get_resume_caption($witem, $m);
            else if ($choice == 'auto_choose')
                $caption = T::g('shell_ext_plugin_button_choose_season_and_episode');
            else if ($choice == 'auto_start')
                $caption = T::g('play_from_the_beginning_button');
            else
                $caption = T::t('button_choose_manually');
            self::add_smart_button($defs, $ndx == 0,
                $id, $caption, $icon, $add_params, $button_gui_params);
        }

        self::add_smart_button($defs, false,
            null, T::g('button_cancel'),
            'gui_skin://button_icons/cancel.aai',
            null, $button_gui_params);

        ControlFactory::add_vgap($defs, 10);

        return ActionFactory::show_dialog(
            T::g('resume_playback_dialog_title'), $defs,
            true, 0);
    }

    private static function find_next_episode($resume_ctx, $m)
    {
        if (!$m->series_type)
            return false;

        $e = HD::get($resume_ctx, 'e', -1);
        if ($e <= 0)
            return false;
        $s = HD::get($resume_ctx, 's', 1);
        $sns = $m->season_numbers;
        $ss = HD::get($sns, $s);
        if (!$ss)
            return false;

        if ($e < $ss->episodes_count) {
            $resume_ctx->e++;
            $resume_ctx->pos = 0;
            return $resume_ctx;
        }

        if ($e == $ss->episodes_count && isset($sns[$s + 1])) {
            $resume_ctx->s++;
            $resume_ctx->e = 1;
            $resume_ctx->pos = 0;
            return $resume_ctx;
        }

        return false;
    }

    private static function check_resume($witem, $m)
    {
        if (!$witem)
            return false;

        $th = MainHelper::$torrent_history;
        $e = HD::get($witem, 'e', -1);
        $s = HD::get($witem, 's', $e >= 0 ? 1 : -1);
        $wh_uid = HD::get($witem, 'uid');
        $sdobj = $th->parse_sup_data(
            HD::get($witem, 'sup_data', ''));
        $autostart = HD::get($sdobj, 'autostart', 0);
        $resume_ctx = (object) array(
            's' => $s, 'e' => $e,
            'pos' => $witem->pos, 'len' => $witem->len,
            'autostart' => $autostart);

        if ($witem->pos < $witem->len - 30)
            return $resume_ctx;
        return self::find_next_episode($resume_ctx, $m);
    }

    private function choose_season_episode_in_shell_ext(
        $user_input, $user_choice, $ignore_wh,
        $chosen_s, $chosen_e)
    {
        $cb = ActionFactory::action_from_user_input(
            $user_input,
            array(
                'control_id' => "user_choice_$user_choice",
                'chosen_se_provided' => 1,
                'ignore_wh' => $ignore_wh,
                'user_choice' => $user_choice));

        $aparams = array(
            'handler_id' => 'entry',
            'control_id' => 'choose_season_episode',
            'result_action' => json_encode($cb),
            'chosen_s' => $chosen_s,
            'chosen_e' => $chosen_e,
        );

        $st = MainHelper::$state;
        $mts = $st->nav_mts;
        $seasons = $mts->get_torrent_seasons();
        $ndx = array_search(0, $seasons);
        if (false !== $ndx)
            unset($seasons[$ndx]);
        if ($seasons)
            $aparams['season_filter'] = implode(',', $seasons);

        return ActionFactory::handle_user_input(
            $aparams, null, null, 'shell_ext');
    }

    private function do_play(&$user_input, $user_choice=null)
    {
        $th = MainHelper::$torrent_history;
        if ($th->dirty)
        {
            hd_print("DEBUG torrent history dirty");
            $cb = ActionFactory::action_from_user_input($user_input);
            return $this->update_wh($cb, true, true);
        }

        $tv_info_updated = HD::get($user_input, 'tv_info_updated');
        if (!$tv_info_updated &&
            defined('UPDATE_TV_INFO_ACTION_ID'))
        {
            $cb = ActionFactory::action_from_user_input(
                $user_input, array('tv_info_updated' => 1));
            return ActionFactory::update_tv_info($cb);
        }

        $ss = MainHelper::$settings;
        $ss->load_tv_info();
        $ss->apply_if_dirty();

        $st = MainHelper::$state;
        $mts = $st->nav_mts;
        $m = $mts->get_movie();

        $autostart = $mts->get_autostart();

        $passed_e = HD::get($user_input, 'chosen_e', -1);
        $passed_s = HD::get($user_input, 'chosen_s', -1);
        $eps_wh_updated = HD::get($user_input, 'eps_wh_updated');

        $se_chosen_manually = HD::get($user_input, 'se_chosen_manually');
        if ($m->series_type && $user_choice == 'resume' &&
            $se_chosen_manually)
        {
            hd_print("Set user_choice resume -> auto_choose for se_chosen_manually");
            $user_choice = 'auto_choose';
            unset($user_input->supplier_info);
        }

        if ($m->series_type && $user_choice == 'auto_choose' &&
            $passed_e >= 0 && !$eps_wh_updated)
        {
            $cb = ActionFactory::action_from_user_input(
                $user_input,
                array('eps_wh_updated' => 1));
            return $this->update_eps_wh($m->id,
                $passed_s, $passed_e, $cb);
        }

        $sup_info = HD::get($user_input, 'supplier_info');
        $sup_info = $sup_info ? json_decode($sup_info) : null;
        $tfile_index = -1;
        $wh_s = -1;
        $wh_e = -1;
        $torrent_uid = null;
        $tfile_id = null;
        $tfile_pos = -1;
        $chosen_op = 'enter';
        $resume_ctx = false;
        $chosen_s = -1;
        $chosen_e = -1;

        if ($sup_info && !$se_chosen_manually)
        {
            // Launching from Recent folder.

            $tfile_index = HD::get($sup_info, 'wh_file_index', -1);
            $wh_uid = HD::get($sup_info, 'wh_uid');
            $wh_s = HD::get($sup_info, 'wh_s', -1);
            $wh_e = HD::get($sup_info, 'wh_e', -1);
            $uobj = $th->parse_file_uid($wh_uid);
            if ($uobj)
            {
                $torrent_uid = $uobj->torrent_uid;
                if ($tfile_index >= 0)
                    $tfile_id = $uobj->file_id;
            }
            $sdobj = $th->parse_sup_data(
                HD::get($sup_info, 'sup_data', ''));
            $autostart = HD::get($sdobj, 'autostart', $autostart);

            $user_choice = 'resume_recent';

            if ($autostart) {
                $user_choice = 'resume';
                $chosen_s = $wh_s;
                $chosen_e = $wh_e;
                $tfile_pos = HD::get($sup_info, 'wh_pos');
                $chosen_op = 'play';
            }
        }
        else
        {
            $witem = null;
            $ignore_wh = HD::get($user_input, 'ignore_wh', 0);
            if ($ignore_wh)
                hd_print("DEBUG ignoring watch history");
            else
            {
                if (($user_choice == 'auto_choose' || $user_choice == 'resume') && 
                    $m->series_type && $passed_e >= 0)
                {
                    $ep = $th->make_ep_tag($m->id, $passed_s, $passed_e);
                    $witem = HD::get($th->witem_by_ep, $ep);
                    if ($witem && $user_choice == 'auto_choose')
                    {
                        $user_choice = null;
                        hd_print("Play: forcing resume/start question ".
                            "for S$passed_s: E$passed_e");
                    }
                    else if ($witem)
                        hd_print("Play: continue resume for S$passed_s: E$passed_e");
                }
                if (!$witem)
                    $witem = $th->get_last_witem($m->id);
            }
            $resume_ctx = self::check_resume($witem, $m);
            hd_print("DEBUG resume_ctx: ".print_r($resume_ctx,1));

            // P possible choices:
            // - resume (assuming have wh item for this movie): first,
            //   doing the same as when entering from recent, but in the
            //   end we launching playback suppressing resume question
            //   dialog
            // - auto_start: select first torrent (of S1E1 if series) and
            //   launch it from beginning
            // - auto_choose: user chooses season, then user choose
            //   episode, then we auto-select first suitable torrent and
            //   launch it
            // - manual (original behaviour): manually choose season,
            //   torrent and file

            if (!$user_choice) {
                $choices = array();
                if ($resume_ctx && $autostart)
                    $choices[] = 'resume';
                if ($resume_ctx && $autostart && $passed_e >= 0)
                    $choices[] = 'auto_start';
                else
                {
                    if ($autostart && !$m->series_type)
                        $choices[] = 'auto_start';
                    if ($autostart && $m->series_type) {
                        if ($resume_ctx)
                            $choices[] = 'auto_start';
                        $choices[] = 'auto_choose';
                    }
                }
                if (!$autostart)
                    $choices[] = 'manual';

                hd_print("DEBUG choices: ".print_r($choices,1));

                if (count($choices) == 1)
                    $user_choice = $choices[0];
                else {
                    return $this->make_user_choice_on_enter(
                        $user_input, $choices, $witem, $m);
                }
            }

            if (!$user_choice)
                return null;

            if ($user_choice == 'resume')
            {
                if (!$resume_ctx)
                    $resume_ctx = self::check_resume($witem, $m);

                $tfile_index = HD::get($witem, 'index', -1);
                $wh_uid = HD::get($witem, 'uid');
                $wh_s = $resume_ctx->s;
                $wh_e = $resume_ctx->e;
                $chosen_s = $resume_ctx->s;
                $chosen_e = $resume_ctx->e;
                $uobj = $th->parse_file_uid($wh_uid);
                if ($uobj)
                {
                    $torrent_uid = $uobj->torrent_uid;
                    $tfile_id = $uobj->file_id;
                    $tfile_pos = $resume_ctx->pos;
                }
                $chosen_op = 'play';
            }
            else if ($user_choice == 'auto_start' && !$m->series_type)
            {
                $chosen_op = 'play';

                $torrent_uid = $mts->get_recent_or_first_torrent_uid(-1, -1);
            }
            else if ($user_choice == 'auto_choose' || $user_choice == 'auto_start')
            {
                if ($user_choice == 'auto_start' && $passed_e < 0) {
                    $chosen_s = 'initial';
                    $chosen_e = 'initial';
                    $chosen_op = 'play';
                    $ignore_wh = 1;
                } else {
                    if ($user_choice == 'auto_start') {
                        hd_print("Play: play from beginning for S$passed_s: E$passed_e");
                        $user_choice = 'auto_choose';
                    }
                    $chosen_s = $passed_s;
                    $chosen_e = $passed_e;
                    $ignore_wh = 0;
                }

                if ($m->series_type &&
                    !HD::get($user_input, 'chosen_se_provided'))
                {
                    return $this->choose_season_episode_in_shell_ext(
                        $user_input, 'auto_choose', $ignore_wh,
                        $chosen_s, $chosen_e);
                }
            }
            else // 'manual'
            {
                $chosen_s = $passed_s;
                $chosen_e = $passed_e;
            }
        }

        if ($user_choice == 'resume' && $m->series_type &&
            !HD::get($user_input, 'chosen_se_provided'))
        {
            return $this->choose_season_episode_in_shell_ext(
                $user_input, 'resume', 0,
                $chosen_s, $chosen_e);
        }

        hd_print("DEBUG choice: $user_choice, S: $chosen_s, E: $chosen_e");

        $choice_auto = in_array($user_choice,
            array('resume', 'auto_choose', 'auto_start'));

        if ($choice_auto)
        {
            if ($chosen_s >= 0 && $chosen_e >= 0) {
                $st->enter_season_and_episode($chosen_s, $chosen_e);
            }

            if ($user_choice != 'resume') {
                $torrent_uid = $mts->get_recent_or_first_torrent_uid(
                    $chosen_s, $chosen_e);
                if (!$torrent_uid)
                {
                    return ActionFactory::show_error(false,
                        T::g('shell_ext_plugin_item_search_no_files_found'));
                }
            }
        }

        $tor = $torrent_uid ? $mts->get_torrent_by_uid($torrent_uid) : null;
        if ($tor)
            hd_print("DEBUG using torrent: ".print_r($tor,1));

        if ($tor && $choice_auto)
        {
            $st->prepare_to_play_torrent();

            $pb = MainHelper::$playback;
            $pb->starting_new_playback();
            $stat = MainHelper::$pb_stat;
            $stat->set_switch_reason('');

            $stack = $user_choice != 'auto_choose' ? '' :
                "S:$chosen_s;E:$chosen_e;";
            hd_print("DEBUG stack: $stack");

            $vod_movie = TorrentHelper::get_all_episodes_vod_movie(
                $stack, $mts->get_autostart(),
                $tor->id, $tor->uid, $chosen_e);

            $sel_id = !$m->series_type ? 'single' :
                ($chosen_e > 0 ? "E$chosen_e" : null);

            hd_print("DEBUG resume position: $tfile_pos");

            $cfg = MainHelper::$config;
            $vod_info = $vod_movie->get_vod_info(
                $sel_id, $tfile_pos, 1);
            return ActionFactory::vod_play($vod_info);
        }

        $enter_torrent_action = null;
        $torrent_sel_id = null;
        if ($tor)
        {
            $torrent_sel_id = TorrentHelper::get_torrent_item_media_url(
                $wh_s, $tor->id);
            $params = array(
                'chosen_media_url' => $torrent_sel_id,
                'chosen_op' => $chosen_op);
            if ($tfile_id) {
                $params['chosen_file_id'] = $tfile_id;
                $params['chosen_file_pos'] = $tfile_pos;
            }
            $enter_torrent_action = MainHelper::$torrents_screen->custom_action(
                'enter', $params);
        }

        $post_action = null;
        $sel_id = null;

        if ($m->series_type)
        {
            $media_url_str = SeasonsScreen::get_media_url_str($m->id);

            $seasons = $mts->get_torrent_seasons();
            if ($tor && in_array($wh_s, $seasons)) {
                $sel_id = $wh_s;
                $post_action = MainHelper::$seasons_screen->custom_action(
                    'enter', array(
                        'chosen_media_url' => $sel_id,
                        'chosen_e' => $wh_e,
                        'torrent_sel_id' => $torrent_sel_id,
                        'enter_torrent_action' =>
                        json_encode($enter_torrent_action)));
            }

            if (!$torrent_uid && $passed_e >= 0 && $passed_s >= 0 &&
                in_array($passed_s, $seasons))
            {
                $sel_id = $passed_s;
                $tor = $mts->get_torrent_by_episode($passed_s, $passed_e);
                $torrent_sel_id = !$tor ? null :
                    TorrentHelper::get_torrent_item_media_url($passed_s, $tor->id);
                $post_action = MainHelper::$seasons_screen->custom_action(
                    'enter', array(
                        'chosen_media_url' => $sel_id,
                        'chosen_e' => $passed_e,
                        'torrent_sel_id' => $torrent_sel_id));
            }
        }
        else
        {
            $st = MainHelper::$state;
            $media_url_str = TorrentsScreen::get_media_url_str(
                $m->id, -1, -1, true, '');

            $sel_id = $torrent_sel_id;
            $post_action = $enter_torrent_action;
        }

        return ActionFactory::open_folder_ext($media_url_str,
            T::t('title_plugin'), null, $sel_id, $post_action);
    }
    
    private function post_update_wh(&$user_input)
    {
        $th = MainHelper::$torrent_history;
        $path = HD::get($user_input, 'result_file');

        $witems = null;
        if (HD::get($user_input, 'action_status') == 1 &&
            $path && is_file($path))
        {
            $doc = file_get_contents($path);
            unlink($path);
            $witems = $doc ? json_decode($doc) : null;
        }

        if (HD::get($user_input, 'eps_mode'))
            $th->set_eps($witems);
        else
            $th->set($witems);

        $cb = HD::get($user_input, 'post_action');
        $cb = $cb ? json_decode($cb, true) : null;
        return $cb;
    }

    public function torrserver_not_started($stop_playback=false) {
        return ActionFactory::show_error(false,
            T::t('errmsg_online_movies_not_available'),
            array("IP: " . TorrServer::getIp()),
            $stop_playback);
    }

    private function load_ts_info()
    {
        $dirpath = "/flashdata/ts";
        $verpath = "$dirpath/version";

        if (!is_dir($dirpath)) {
            if (!mkdir("$dirpath", 0755)) {
                hd_print("Error: mkdir($dirpath) failed");
                return null;
            }
        }

        $doc = HD::http_get_document(
            'https://releases.yourok.ru/torr/server_release.json');
        $jsd = $doc ? json_decode($doc) : null;
        if (!$jsd || !isset($jsd->version) ||
            !isset($jsd->links->{'android-arm7'}))
        {
            hd_print("Error: failed to install ts: ".
                "invalid releases json");
            return null;
        }

        file_put_contents($verpath, $jsd->version);

        $url = $jsd->links->{'android-arm7'};
        if (0 !== strpos($url, "http://") &&
            0 !== strpos($url, "https://"))
        {
            hd_print("Error: failed to install ts: ".
                "invalid download url");
            return null;
        }

        return (object) array('version' => $jsd->version, 'url' => $url);
    }

    private function do_update_torrserver_settings()
    {
        $settings = TorrServer::torrserverActionSettingsRead();
        if (!$settings || !isset($settings->DisableUpload)) {
            hd_print("Warning: invalid ts settings read");
        } else {
            $settings->DisableUpload = true;
            TorrServer::torrserverActionSettingsWrite((array) $settings);
        }
    }

    private static function is_ts_installed()
    {
        clearstatcache();
        $dirpath = "/flashdata/ts";
        $binpath = "$dirpath/ts";
        return is_file($binpath);
    }

    private function do_install_apk_ts($target_path)
    {
        if (!HD::is_apk())
            return false;

        if (is_file($target_path) || is_link($target_path))
            unlink($target_path);
        $thisdir = DuneSystem::$properties["install_dir_path"];
        symlink("$thisdir/bin/ts", $target_path);
        return true;
    }

    private function do_ensure_own_torrserver()
    {
        clearstatcache();

        $dirpath = "/flashdata/ts";
        $binpath = "$dirpath/ts";
        $logpath = "/tmp/run/ts.log";
        $port = TorrServer::$PORT;
        $setspath = $dirpath;

        if (!is_dir($dirpath)) {
            if (!mkdir("$dirpath", 0755)) {
                hd_print("Error: mkdir($dirpath) failed");
                return false;
            }
        }

        if (!is_file($binpath)) {
            if (!$this->do_install_apk_ts($binpath))
            {
                hd_print("Error: ts bin $binpath not exist unexpectedly");
                return false;
            }
        }

        if (!is_executable($binpath)) {
            if (!chmod($binpath, 755)) {
                hd_print("Error: chmod($dirpath) failed");
                return false;
            }
            if (!is_executable($binpath)) {
                hd_print("Error: failed to make file executable: $dirpath");
                return false;
            }
        }

        putenv("GODEBUG=madvdontneed=1");

        // NOTE: 'X=Y' is hack to suppress auto adding the 'sh' prefix by
        // apk version of libphp5.so
        $cmd = "X=Y $binpath --port $port -k -d $setspath -l $logpath 1>>$logpath 2>&1 &";
        if (!HD::is_apk())
            $cmd = "/system/dunehd/dunehd_service do $cmd";
        system($cmd, $ret);
        hd_print("Waiting for TorrServer start ($ret)...");

        for ($i = 0; $i < 25; $i++)
        {
            if ($i > 0)
                hd_print("Still waiting...");

            usleep(400000);
            if (TorrServer::getOnlineStatusServer()) {
                hd_print("TorrServer status: OK");
                return true;
            }
        }

        hd_print("Timed out waiting for TorrServer start");
        return false; 
    }

    public function ensure_own_torrserver($user_input, $result_action)
    {
        $ok = TorrServer::getOnlineStatusServer();
        if (!$ok) {
            hd_print("TorrServer not started, ip: " . TorrServer::getIp());

            if (!MainHelper::$settings->ext_torrserver)
            {
                if (!HD::is_apk() && !self::is_ts_installed())
                    return $this->install_ts($user_input, $result_action);

                $ok = $this->do_ensure_own_torrserver();
                if ($ok)
                    $this->do_update_torrserver_settings();
            }
        }

        if (!$ok) {
            $p = MainHelper::$progress;
            return $p->close_and_run($user_input,
                $this->torrserver_not_started());
        }

        return $result_action;
    }

    private function update_video_streams_osd_menu()
    {
        $mts = MainHelper::$state->play_mts;
        $season = $mts->get_season();
        $tfs = MainHelper::$state->play_tfs;
        $torrent_id = $tfs ? $tfs->get_torrent_id() : null;

        $sel_ndx = -1;
        $mitems = array();
        foreach ($mts->get_torrents() as $ndx => $t) {
            $inf = $mts->get_torrent_info($t->id);

            $marked = $t->id == $torrent_id;
            if ($marked)
                $sel_ndx = count($mitems);

            $pub_ndx = $ndx + 1;
            $caption = $pub_ndx . ". " . $inf->get_caption($season);
            $extra_text = HD::get($t, 'extra_item');                               
            if ($extra_text)                                                       
                $caption .= " | " . $extra_text;
            $mitems[] = ActionFactory::get_osd_menu_item(
                $t->id, $caption, $marked);
        }
        
        $menu_params = array('font_size' => 'small');

        return ActionFactory::add_menu_items(
            'video_streams', $mitems, $sel_ndx, $menu_params);
    }

    public function enter_osd_menu(&$user_input)
    {
        $id = HD::get($user_input, 'osd_menu_id');
        if (!isset($id))
            $id = 'video_streams';

        if ($id == 'video_streams')
            return $this->update_video_streams_osd_menu();

        MainHelper::$pb_stat->set_switch_reason('manual_switch');

        $torrent_id = $id;

        $mts = MainHelper::$state->play_mts;
        $tfs = MainHelper::$state->play_tfs;

        $t = $mts->get_torrent_by_id($torrent_id);
        if (!$t)
        {
            hd_print("enter_osd_menu E: torrent not found by id $id");
            return null;
        }

        $pub_ndx = $mts->get_torrent_public_index($torrent_id);
        hd_print("Entering torrent $pub_ndx: $t->name");

        $cur_t = $mts->get_torrent_by_id($tfs->get_torrent_id());
        if ($cur_t && $cur_t->id == $t->id)
        {
            hd_print("Torrent not changed");
            return null;
        }

        $pb = MainHelper::$playback;
        $e = $pb->get_episode();

        $start_action = MainHelper::$progress->custom_action(
            'switch_open_torrent',
            array('e' => $e, 'torrent_id' => $t->id));

        $swd = MainHelper::$switch_dialog;
        return $swd->open(false, $start_action);
    }

    private function update_timer($user_input)
    {
        // TODO hack?
        $parent_murl = MediaURL::decode($user_input->parent_media_url);

        if (HD::get($parent_murl, 'screen_id') == TorrentsScreen::ID)
        {
            return MainHelper::$torrents_screen->speed_up_timer(
                $parent_murl, $plugin_cookies);
        }
        if (HD::get($parent_murl, 'screen_id') == TorrentFilesScreen::ID)
        {
            return MainHelper::$torrent_files_screen->speed_up_timer(
                $parent_murl, $plugin_cookies);
        }

        return null;
    }

    private function download_torrent(&$user_input, &$plugin_cookies)
    {
        $st = MainHelper::$state;
        $mts = $st->nav_mts;
        $tfs = $st->nav_tfs;
        $torrent_id = $user_input->torrent_id;
        $t = $mts->get_torrent_by_id($torrent_id);
        $series = $tfs->get_series();

        $single_path = HD::get($user_input, 'file_path');
        $files = array();
        foreach ($series as $path => $ser)
        {
            if ($single_path && $path != $single_path)
                continue;

            $url = $ser['playback_url'];
            $size_mb = HD::get($ser, 'size_mb', -1);
            DlState::add_download_file($files,
                $path, $url, $size_mb);
        }

        $n = count($files);
        hd_print("Download: $t->name, $n files");

        DlState::add_download($t->group_id, $t->name, $files);

        return $this->update_timer($user_input);
    }

    private function delete_torrent_data(&$user_input, &$plugin_cookies)
    {
        $torrent_id = $user_input->torrent_id;
        $file_path = HD::get($user_input, 'file_path');

        $st = MainHelper::$state;
        $mts = $st->nav_mts;
        $t = $mts->get_torrent_by_id($torrent_id);
        if (!$t)
            return null;

        hd_print("Delete: $t->name");

        $file_paths = $file_path ? array($file_path) : null;
        DlState::remove_download($t->group_id, $file_paths);

        return $this->update_timer($user_input);
    }

    private function show_in_folder(&$user_input, &$plugin_cookies)
    {
        $torrent_id = $user_input->torrent_id;
        $file_path = HD::get($user_input, 'file_path');

        $sh = MainHelper::$storage_helper;
        $storage_name = $sh->get_dl_storage_name();
        if (!$storage_name)
        {
            return ActionFactory::show_error(false,
                T::g('shell_ext_plugin_title_storage_not_attached'));
        }

        $dir_path = "dune_downloads";
        $dir = HD::path_dirname($file_path);
        if (strlen($dir) && $dir != '/')
            $dir_path .= "/$dir";

        $fname = HD::path_basename($file_path);

        return ActionFactory::launch_media_url(
            "embedded_app://{name=path_from_source}".
            "{source_type=storage}".
            "{source_id=$storage_name}".
            "{path=$dir_path}{sel_id=$fname}");
    }

    public function apply_ltsb($user_input)
    {
        $dirpath = "/flashdata/ts";
        $binpath_l = "$dirpath/ts.loading";
        $binpath = "$dirpath/ts";

        $st = MainHelper::$state;
        $st->installing_ts = 0;

        $status = HD::get($user_input, 'ltsb_status');
        if ($status != 'succeeded')
        {
            hd_print("Loading failed => abort");

            if (is_file($binpath_l))
                unlink($binpath_l);

            $p = MainHelper::$progress;
            return ActionFactory::close_and_run(
                $p->close_and_run($user_input,
                    $this->torrserver_not_started()));
        }

        if (is_file($binpath))
            unlink($binpath);
        rename($binpath_l, $binpath);

        if (!$this->do_ensure_own_torrserver())
        {
            $p = MainHelper::$progress;
            $a = $p->close_and_run($user_input,
                $this->torrserver_not_started());
        }
        else
            $a = ActionFactory::decode_result_action($user_input);
        return ActionFactory::close_and_run($a);
    }

    public function handle_user_input(&$user_input, &$plugin_cookies)
    {
        $control_id = HD::get($user_input, 'control_id', 'none');

        if ($control_id != 'playback_timer' &&
            $control_id != 'apply_torlist')
        {
            hd_print("Main user_input: ");
            foreach ($user_input as $k => $v) {
                if ($k == 'movie_str')
                    $v = strlen($v) . " bytes";
                else if (strlen($v) > 1010) {
                    $n = strlen($v) - 1000;
                    $v = substr($v, 0, 1000) . "... ($n bytes more)";
                }
                hd_print("  $k: $v");
            }
        }

        if ($control_id == 'play')
            return $this->play($user_input);

        if ($control_id == 'do_play') {
            $user_choice = HD::get($user_input, 'user_choice');
            return $this->do_play($user_input, $user_choice);
        }

        if (0 === strpos($control_id, 'user_choice_')) {
            $user_choice = substr($control_id, strlen('user_choice_'));
            return $this->do_play($user_input, $user_choice);
        }

        if ($control_id == 'post_update_wh')
            return $this->post_update_wh($user_input);

        if ($control_id == 'ensure_own_torrserver') {
            return $this->ensure_own_torrserver($user_input,
                ActionFactory::decode_result_action($user_input));
        }

        if ($control_id == 'choose_playback_stream')
            return MainHelper::$playback->choose_playback_stream($user_input);

        if ($control_id == 'apply_torlist')
        {
            // Currently NOP.
            // TODO: revise, maybe remove torrent_list request ?...
            return null;
        }

        if ($control_id == 'enter_osd_menu')
            return $this->enter_osd_menu($user_input);

        if ($control_id == 'download_torrent')
            return $this->download_torrent($user_input, $plugin_cookies);

        if ($control_id == 'delete_torrent_data')
            return $this->delete_torrent_data($user_input, $plugin_cookies);

        if ($control_id == 'show_in_folder')
            return $this->show_in_folder($user_input, $plugin_cookies);

        if ($control_id == 'playback_state_changed')
            return MainHelper::$playback->state_changed($user_input);

        if ($control_id == 'playback_user_action')
            return MainHelper::$playback->user_action($user_input);

        if ($control_id == 'playback_timer')
            return MainHelper::$playback->timer($user_input);

        if ($control_id == 'cancel_dialog')
            return ActionFactory::close_dialog();

        if ($control_id == 'apply_ltsb')
            return $this->apply_ltsb($user_input);

        return null;
    }
}

///////////////////////////////////////////////////////////////////////////
?>
