<?php

require_once 'torrent_info.php';

class MovieTorrents
{
    private $movie;
    private $lang;
    private $region;

    private $all_torrents = array();
    private $torrent_by_id = array();
    private $torrents_by_season = array();

    private $good_ids = array();
    private $inf_by_id = array();

    // Selected 'autostart'.
    private $autostart;

    // Selected season/episode/torrent. Following torrent lists depend on
    // season/episode.
    private $season = -1;
    private $episode = -1;
    private $torrent_id = null;

    // All torrents for selected season/episode.
    private $torrents = array();

    // @torrents reordered by moving several chosen torrents to the
    // begginning of list. These torrents chosen for auto-switch because of
    // playback problems (could not start video or video freezes for
    // buffering for too long or too often)
    private $reordered_torrents = array();

    // First N torrents from @reordered_torrents list to be shown on separate
    // screen.
    private $chosen_torrents = array();

    private $titles = array();
    private $trimmed_titles = array();

    private $last_view_index = -1;

    private $debug = false;

    private static $SPECIAL_COUNTRY_BY_LANG = array(
        'ru' => 'RU',
        'uk' => 'RU',
    );

    public function __construct($movie, $lang, $region) {
        $this->movie = $movie;
        $this->lang = $lang;
        $this->region = $region;

        $this->debug = is_file("/config/omov_debug");
    }

    public function make_copy()
    {
        $mts = new MovieTorrents($this->movie, $this->lang, $this->region);

        $mts->all_torrents = $this->all_torrents;
        $mts->torrent_by_id = $this->torrent_by_id;
        $mts->torrents_by_season = $this->torrents_by_season;

        $mts->good_ids = $this->good_ids;
        $mts->inf_by_id = $this->inf_by_id;

        $mts->autostart = $this->autostart;
        $mts->season = $this->season;
        $mts->episode = $this->episode;
        $mts->torrent_id = $this->torrent_id;

        $mts->torrents = $this->torrents;
        $mts->reordered_torrents = $this->reordered_torrents;
        $mts->chosen_torrents = $this->chosen_torrents;

        $mts->titles = $this->titles;
        $mts->trimmed_titles = $this->trimmed_titles;
        $mts->last_view_index = $this->last_view_index;

        return $mts;
    }

    ///////////////////////////////////////////////////////////////////////

    public static function cmp_seasons($s1, $s2)
    {
        if ($s1 == 0)
            return $s2 == 0 ? 0 : 1;
        if ($s2 == 0)
            return -1;
        return $s1 - $s2;
    }

    private static $cmp_torrents_mts;

    private static function check_user_prefs($q, $s)
    {
        $ss = MainHelper::$settings;
        $n = 0;

        if ($s < $ss->min_s)
            $n++;

        if ($q < $ss->min_q || $q > $ss->max_q)
            $n++;

        return $n;
    }

    private static function s_coef($s, $q)
    {
        $qq = $q >= TorrentInfo::QRES_2160P ? 4 :
            ($q >= TorrentInfo::QRES_1080P ? 3 :
             ($q >= TorrentInfo::QRES_720P ? 2 : 1));
        $ss = $s <= 0 ? 0 :
            ($s <= 4 ? $s :
             ($s < 10 ? 5 :
              ($s < 20 ? 10 :
               ($s < 30 ? 20 :
                ($s < 50 ? 30 :
                 ($s < 100 ? 50 : 100))))));
        return $qq * $ss;
    }

    public static function cmp_torrents($t1, $t2)
    {
        $st = MainHelper::$state;
        $mts = self::$cmp_torrents_mts;

        $inf1 = $mts->inf_by_id[$t1->id];
        $inf2 = $mts->inf_by_id[$t2->id];
        if ($inf1->bonus != $inf2->bonus)
            return $inf1->bonus ? 1 : -1;
        if ($inf1->invalid != $inf2->invalid)
            return $inf1->invalid ? 1 : -1;

        $g1 = isset($mts->good_ids[$t1->id]) ? 1 : 0;
        $g2 = isset($mts->good_ids[$t2->id]) ? 1 : 0;
        if ($g1 != $g2)
            return $g2 - $g1;

        $m1 = self::check_user_prefs($inf1->res_quality, $t1->S);
        $m2 = self::check_user_prefs($inf2->res_quality, $t2->S);
        if ($m1 != $m2)
            return $m1 - $m2;

        $svs = MainHelper::setting('sort_video_streams');
        if ($svs == 'quality') {
            // Nop
        } else if ($svs == 'availability') {
            $s1 = $t1->S;
            $s2 = $t2->S;
            if ($s1 != $s2)
                return $s2 - $s1;
        }
        else
        {
            $s1 = self::s_coef($t1->S, $inf1->res_quality);
            $s2 = self::s_coef($t2->S, $inf2->res_quality);
            if ($s1 != $s2)
                return $s2 - $s1;
        }

        if ($inf1->quality != $inf2->quality)
            return $inf2->quality - $inf1->quality;

        if ($t1->S != $t2->S)
            return $t2->S - $t1->S;

        if ($t1->size_mb != $t2->size_mb)
            return $t1->size_mb > $t2->size_mb ? -1 : 1;

        $n = strnatcasecmp($t1->name, $t2->name);
        if ($n != 0)
            return $n;

        return strcmp($t1->id, $t2->id);
    }

    private function request_torrents($title, $year,
        &$idset, $dump_fp, $ndx)
    {
        return MainHelper::$tordb->request_torrents(
            $title, $year, $idset, $dump_fp, $ndx,
            $this->lang, $this->region,
            $this->all_torrents, $this->good_ids);
    }

    public function make_titles($silent=false)
    {
        $m = $this->movie;

        $title1 = $m->native_title ? $m->native_title : $m->title;
        $this->titles = array($title1);
        if ($title1 != $m->title)
            $this->titles[] = $m->title;

        if (count($this->titles) == 2)
        {
            $countries = explode(',', $m->country_codes);
            $country = HD::get(self::$SPECIAL_COUNTRY_BY_LANG, $this->lang);
            if ($country && in_array($country, $countries))
            {
                if (!$silent)
                    hd_print("DEBUG: reverting titles order");
                $this->titles = array($m->title, $title1);
            }
        }

        $this->trimmed_titles = array();
        foreach ($this->titles as $title)
            $this->trimmed_titles[] = TorrentInfo::trim_special_chars($title);

        return $this->trimmed_titles;
    }

    private function set_autostart($autostart)
    {
        $this->autostart = $autostart >= 0 ? $autostart :
            MainHelper::$config->autostart_enabled();
    }

    private function filter_all_torrents_by_lang()
    {
        $n1 = count($this->all_torrents); // EEE
        LangHelper::filter_torrent_list(
            $this->all_torrents, $this->good_ids);
        $n2 = count($this->all_torrents);
    }

    public function open($autostart=-1)
    {
        $this->all_torrents = array();
        $this->torrent_by_id = array();
        $this->torrents_by_season = array();
        $this->good_ids = array();
        $this->inf_by_id = array();

        $this->set_autostart($autostart);

        $this->season = -1;
        $this->episode = -1;
        $this->torrent_id = null;
        $this->clear_tlists();

        $m = $this->movie;

        $this->make_titles();

        $pname = DuneSystem::$properties['plugin_name'];
        $DPATH = "/flashdata/plugins_data/$pname/torlog";
        $dump_fp = false;
        if (is_dir($DPATH)) {
            $path = "$DPATH/{$this->trimmed_titles[0]}.$m->year.$m->id.txt";
            $dump_fp = fopen($path, "w");
        }

        $idset = array();
        $ndx = 0;

        /**
         * NOTE: we search for single movies with title+year, but only with
         * 'title' for seasons. Further validation is performed allowing
         * +/-1 year related to movie year or season year, if season has
         * been parsed from torrent name; invalid torrents listed after
         * valid.
         */

        //$search_with_year = !$m->series_type || count($m->season_numbers) == 1;

        $search_with_year = MainHelper::$tordb->support_search_with_year();

        if ($search_with_year) {
            foreach ($this->titles as $title) {
                $ndx = $this->request_torrents($title,
                    $m->year, $idset, $dump_fp, $ndx);
            }
        }

        if (!$search_with_year || count($this->all_torrents) == 0) {
            foreach ($this->titles as $title) {
                $ndx = $this->request_torrents($title,
                    null, $idset, $dump_fp, $ndx);
            }
        }

        if ($dump_fp)
            fclose($dump_fp);

        $this->filter_all_torrents_by_lang();

        $this->ensure_torrents_info(true);
        $this->filter_all_torrents();
        $this->make_torrent_by_id_map();
        $this->make_torrents_by_season();

        $this->make_torrents();
        $this->make_reordered_torrents();

        MainHelper::$dl_state->reload_gstat();

        return $this->all_torrents;
    }

    public function apply_sort_change()
    {
        usort($this->all_torrents, 'MovieTorrents::cmp_torrents');

        $this->torrent_by_id = array();
        $this->torrents_by_season = array();
        $this->clear_tlists();

        $this->filter_all_torrents();
        $this->make_torrent_by_id_map();
        $this->make_torrents_by_season();

        $this->make_torrents();
        $this->make_reordered_torrents();
    }

    private function clear_tlists() {
        $this->torrents = array();
        $this->reordered_torrents = array();
        $this->chosen_torrents = array();
    }

    public function reopen($autostart=-1)
    {
        $this->set_autostart($autostart);
        $this->set_season(-1);
    }

    public function set_season($season)
    {
        $this->set_season_and_episode($season, -1);

        MainHelper::$dl_state->reload_gstat();
    }

    public function set_episode($episode)
    {
        if ($episode == $this->episode)
            return;

        $this->episode = $episode;
        $this->torrent_id = null;
        $this->make_torrents();
        $this->make_reordered_torrents();
    }

    public function set_season_and_episode($s, $e)
    {
        if ($s == $this->season && $e == $this->episode)
            return;

        $this->season = $s;
        $this->episode = $e;
        $this->torrent_id = null;
        $this->make_torrents();
        $this->make_reordered_torrents();

        MainHelper::$dl_state->reload_gstat();
    }

    public function set_torrent_id($torrent_id) {
        $this->torrent_id = $torrent_id;
    }

    private function make_torrents_by_season()
    {
        if (!$this->movie->series_type)
            return;

        foreach ($this->all_torrents as $t)
        {
            $inf = HD::get($this->inf_by_id, $t->id);
            if (!$inf || $inf->bonus || $inf->name_or_year_invalid)
                $this->torrents_by_season[0][] = $t;
            else if ($inf->is_single_season())
                $this->torrents_by_season[1][] = $t;
            else if (!$inf->seasons)
                $this->torrents_by_season[0][] = $t;
            else {
                foreach ($inf->seasons as $s) {
                    $this->torrents_by_season[$s][] = $t;
                }
            }
        }

        uksort($this->torrents_by_season, 'MovieTorrents::cmp_seasons');
    }

    private function filter_all_torrents()
    {
        // XXX TODO: revise
//        if ($this->debug)
//            return;

        if ($this->movie->series_type)
            return;
        return;

        // Filtering out some torrents.
        // Currently they are: 'N' (invalid name), 'E' (existing but
        // invalid year), 'T' (contains some bad tags and no good tags)
        //
        // TODO: maybe add some possibility for user to show all torrents
        // (toggle by A_RED? or maybe move them to special folder, like
        // 'Others' for TV-series?)

        $torrents = array();
        foreach ($this->all_torrents as $t) {
            $inf = $this->inf_by_id[$t->id];
            if (!$inf->name_or_year_invalid)
                $torrents[] = $t;
        }
        $num_filtered = count($torrents);
        $total = count($this->all_torrents);
        hd_print("DEBUG: filter_all_torrents: $total -> $num_filtered");
        $this->all_torrents = $torrents;
    }

    private function make_torrent_by_id_map()
    {
        foreach ($this->all_torrents as $t) {
            $this->torrent_by_id[$t->id] = $t;
        }
    }

    public function ensure_torrents_info($force)
    {
        $series = $this->movie->series_type ? 1 : 0;
        $vidx = MainHelper::$torrents_screen->get_view_index(
            MainHelper::$plugin->get_plugin_cookies());

        if (!$force && $vidx == $this->last_view_index)
            return;

        if ($vidx != $this->last_view_index)
            hd_print("DEBUG: torrents index changed: $this->last_view_index -> $vidx");

        $remove_movie_names = $vidx == 0 ? 0 : 1;
        $num_invalid = 0;
        foreach ($this->all_torrents as $t)
        {
            $good = isset($this->good_ids[$t->id]);
            $inf = new TorrentInfo($this->trimmed_titles, $remove_movie_names,
                $series, $this->movie->year, $this->movie->season_numbers,
                $good, $t->name, $t->size_mb, 0, 0);
            $inf->parse();

            $this->inf_by_id[$t->id] = $inf;
            if ($inf->invalid)
                $num_invalid++;
        }

        self::$cmp_torrents_mts = $this;
        MainHelper::$settings->apply_if_dirty();
        usort($this->all_torrents, 'MovieTorrents::cmp_torrents');

        $this->last_view_index = $vidx;

        $total = count($this->all_torrents);
        if ($total > 0) {
            $perc = sprintf("%.2f", $num_invalid * 100.0 / $total);
            hd_print("Torrents invalid: $num_invalid of total $total ($perc%)");
        }
    }

    private function make_torrents()
    {
        $s = $this->season;
        $e = $this->episode;
        if ($s < 0) {
            $this->torrents = $this->all_torrents;
        } else if ($e < 0) {
            $this->torrents =
                HD::get($this->torrents_by_season, $s, array());
        } else {
            $this->torrents = array();
            $season_torrents = 
                HD::get($this->torrents_by_season, $s, array());
            $n = count($season_torrents);
            foreach ($season_torrents as $t) {
                $inf = HD::get($this->inf_by_id, $t->id);
                if (!$inf)
                    continue;

                if ($inf->has_episode($s, $e))
                    $this->torrents[] = $t;
            }
        }

        $n = count($this->torrents);
        $all = count($this->all_torrents);
        hd_print("DEBUG make_torrents(S$s E$e): $n matched of $all total");
    }

    private function make_reordered_torrents()
    {
        $this->chosen_torrents = array();
        $this->reordered_torrents = array();

        $svs = MainHelper::setting('sort_video_streams');
        if ($svs == 'quality')
        {
            $qres_arr = array(
                TorrentInfo::QRES_2160P,
                TorrentInfo::QRES_1080P,
                TorrentInfo::QRES_720P,
                0);
            $ntors_by_qres = array();
            foreach ($qres_arr as $qres)
                $ntors_by_qres[$qres] = 0;

            foreach ($this->torrents as $t)
            {
                $inf = $this->get_torrent_info($t->id);
                $bad = $inf->invalid || !$this->is_good_torrent($t->id);
                if ($bad)
                    continue;

                $q = $inf->res_quality;
                foreach ($qres_arr as $qres)
                {
                    if ($q < $qres)
                        continue;

                    $ntors_by_qres[$qres]++;
                    if ($ntors_by_qres[$qres] > 2)
                        break;

                    $this->chosen_torrents[] = $t;
                    break;
                }

                if (count($this->chosen_torrents) >= 8)
                    break;
            }
        }

        if (count($this->chosen_torrents) == 0)
            $this->chosen_torrents = array_slice($this->torrents, 0, 10);

        $set = array();
        foreach ($this->chosen_torrents as $t)
        {
            $set[$t->id] = 1;
            $this->reordered_torrents[] = $t;
        }

        foreach ($this->torrents as $t) {
            if (!isset($set[$t->id]))
                $this->reordered_torrents[] = $t;
        }
    }

    public function get_torrent_index($torrent_id, $special_order=true)
    {
        $tlist = $special_order ? $this->reordered_torrents : $this->torrents;
        return self::find_torrent_by_id($tlist, $torrent_id);
    }

    public function get_torrent_public_index($torrent_id)
    {
        return 1 + self::find_torrent_by_id($this->torrents, $torrent_id);
    }

    public function get_next_torrent($torrent_id, $special_order=true)
    {
        $tlist = $special_order ? $this->reordered_torrents : $this->torrents;
        $ndx = self::find_torrent_by_id($tlist, $torrent_id);
        if ($ndx < 0) {
            // Should not happen.
            hd_print("get_next_torrent W: torrent not found");
            return null;
        }

        $total = count($tlist);
        $ndx = $ndx == $total - 1 ? 0 : $ndx + 1;
        return $tlist[$ndx];
    }

    public function get_recent_or_first_torrent_uid(
        $chosen_s=-1, $chosen_e=-1)
    {
        $th = MainHelper::$torrent_history;

        $last_suitable_t = null;
        $last_start = null;
        $last_ndx = -1;

        foreach ($this->torrents as $ndx => $t)
        {
            $inf = HD::get($this->inf_by_id, $t->id);
            if (!$inf)
                continue;

            if ($chosen_e >= 0)
            {
                if (!$inf->has_episode($chosen_s, $chosen_e))
                    continue;
            }

            $tag = $th->make_torrent_tag($t->uid);
            $witem = HD::get($th->witem_by_tag, $tag);
            if (!$witem || $witem->start <= 0)
                continue;

            if ($last_start && $witem->start <= $last_start)
                continue;

            $last_start = $witem->start;
            $last_suitable_t = $t;
            $last_ndx = $ndx;
        }

        $for_log = $chosen_e < 0 ?
            "movie" : "S$chosen_s, E$chosen_e";
        $total = count($this->torrents);

        if ($last_suitable_t) {
            $n = $last_ndx + 1;
            hd_print("Chosen torrent $n of $total for $for_log");
            return $last_suitable_t->uid;
        }

        $chosen = $this->chosen_torrents;
        if (!$chosen)
            return null;

        hd_print("Chosen first torrent of $total for $for_log");
        return $chosen[0]->uid;
    }

    public function get_torrent_by_episode($s, $e)
    {
        $torrents = HD::get($this->torrents_by_season, $s, array());
        foreach ($torrents as $ndx => $t) {
            if ($ndx >= 10)
                break;
            $inf = HD::get($this->inf_by_id, $t->id);
            if (!$inf)
                continue;
            if ($inf->has_episode($s, $e))
                return $t;
        }
        return null;
    }

    private static function find_torrent_by_id($tlist, $id)
    {
        foreach ($tlist as $n => $t) {
            if ($t->id == $id)
                return $n;
        }
        return -1;
    }

    public function get_torrent_by_id($id)
    {
        return HD::get($this->torrent_by_id, $id);
    }

    public function get_torrent_by_uid($uid)
    {
        foreach ($this->all_torrents as $t)
            if ($t->uid == $uid)
                return $t;
        return null;
    }

    public function get_torrents()
    {
        return $this->torrents;
    }

    public function get_reordered_torrents()
    {
        return $this->reordered_torrents;
    }

    public function get_chosen_torrents()
    {
        return $this->chosen_torrents;
    }

    public function is_chosen_torrent_id($id)
    {
        $ndx = self::find_torrent_by_id($this->chosen_torrents, $id);
        return $ndx != -1;
    }

    public function get_torrent_seasons()
    {
        return array_keys($this->torrents_by_season);
    }

    public function get_torrent_info($torrent_id)
    {
        return $this->inf_by_id[$torrent_id];
    }

    public function is_good_torrent($torrent_id)
    {
        return isset($this->good_ids[$torrent_id]);
    }

    public function get_movie()
    {
        return $this->movie;
    }

    public function get_movie_id()
    {
        return $this->movie->id;
    }

    public function get_season()
    {
        return $this->season;
    }

    public function get_episode()
    {
        return $this->episode;
    }

    public function get_torrent_id()
    {
        return $this->torrent_id;
    }

    public function get_autostart()
    {
        return $this->autostart;
    }
}

?>
