<?php

class PBStat
{
    public $movie_id;
    public $movie_title;
    public $movie_year;
    public $season;
    public $episode;

    public $t_id;
    public $t_name;
    public $t_q;
    public $t_size_mb;
    public $t_S;
    public $t_L;

    public $t_file;

    public $start_pos;
    public $last_pos;
    public $end_pos;
    public $duration;

    public $switch_reason;
    public $stop_reason;

    public $before_first_playing;
    public $total_playing;
    public $total_paused;
    public $total_other_buffering;
    public $num_other_bufferings;
    public $num_seeks;
    public $num_pauses;

    public function set_movie($movie_id,
        $movie_title, $movie_year, $season, $episode)
    {
        $this->movie_id = $movie_id;
        $this->movie_title = $movie_title;
        $this->movie_year = $movie_year;
        $this->season = $season;
        $this->episode = $episode;
    }

    public function set_torrent($t_id,
        $t_name, $t_q, $t_size_mb, $t_S, $t_L)
    {
        $this->t_id = $t_id;
        $this->t_name = $t_name;
        $this->t_q = $t_q;
        $this->t_size_mb = $t_size_mb;
        $this->t_S = $t_S;
        $this->t_L = $t_L;
    }

    public function cleanup()
    {
        $this->t_id = null;
        $this->t_name = '';
        $this->t_q = 0;
        $this->t_size_mb = -1;
        $this->t_S = -1;
        $this->t_L = -1;
        $this->t_file = '';

        $this->start_pos = -1;
        $this->last_pos = -1;
        $this->end_pos = -1;
        $this->duration = -1;
        $this->stop_reason = '';
        $this->switch_reason = '';
        $this->before_first_playing = -1;
        $this->total_playing = 0;
        $this->total_paused = 0;
        $this->total_other_buffering = 0;
        $this->num_other_bufferings = 0;
        $this->num_seeks = 0;
        $this->num_pauses = 0;
    }

    public function set_position($pos)
    {
        $pos = (int) round($pos);
        $this->last_pos = $pos;
        $this->end_pos = $pos;
        if ($this->start_pos < 0)
            $this->start_pos = $pos;
    }

    public function set_before_first_playing($tm)
    {
        $this->before_first_playing = round($tm);
    }

    public function set_switch_reason($str)
    {
        hd_print("DEBUG set_switch_reason($str)");
        $this->switch_reason = $str;
    }

    public function state_changed($state, $tm,
        $last_state, $last_state_tm,
        $last_state_special_buffering)
    {
        $tm_diff = (int) round($tm - $last_state_tm);
        if ($last_state == 'playing') {
            $this->total_playing += $tm_diff;
        } else if ($last_state == 'paused') {
            $this->total_paused += $tm_diff;
            $this->num_pauses++;
        } else if ($last_state == 'seeking') {
            $this->num_seeks++;
        } else if ($last_state == 'buffering') {
            if (!$last_state_special_buffering)
                $this->num_other_bufferings++;
        }
    }

    public function playback_stop($duration,
        $is_error, $is_eos, $stop_pressed)
    {
        $this->duration = $duration;
        if ($this->switch_reason)
        {
            hd_print("Using switch_reason as stop_reason: $this->switch_reason");
            $this->stop_reason = $this->switch_reason;
            $this->switch_reason = '';
        }
        else
        {
            $this->stop_reason = $is_error ? "error" :
                ($is_eos ? "eos" : ($stop_pressed ? "stop" : "other"));
        }

        hd_print("DEBUG stop_reason: $this->stop_reason");

        $rec = (object) array(
            'm_id' => $this->movie_id,
            'm_title' => $this->movie_title,
            'm_year' => $this->movie_year,
            's' => $this->season,
            'e' => $this->episode,

            't_id' => $this->t_id,
            't_name' => $this->t_name,
            't_q' => $this->t_q,
            't_size_mb' => $this->t_size_mb,
            't_S' => $this->t_S,
            't_L' => $this->t_L,

            't_file' => $this->t_file,

            'start_pos' => $this->start_pos,
            'end_pos' => $this->end_pos,
            'stop_reason' => $this->stop_reason,
            'duration' => $this->duration,

            'before_first_playing' => $this->before_first_playing,
            'playing' => $this->total_playing,
            'paused' => $this->total_paused,
            'buffering' => $this->total_other_buffering,
            'num_bufferings' => $this->num_other_bufferings,
            'num_seeks' => $this->num_seeks,
            'num_pauses' => $this->num_pauses,
        );
        $str = MediaURL::raw_json_encode($rec);

        $path = DuneSystem::$properties['tmp_dir_path'] . "/play.log";
        clearstatcache(true, $path);
        $st = is_file($path) ? stat($path) : null;
        $size = $st ? $st['size'] : 0;
        if ($size > 15000)
        {
            hd_print("Rotating play.log of size: $size");
            $old_path = $path . ".old";
            rename($path, $old_path);
        }

        file_put_contents($path, $str . "\n", FILE_APPEND);

        $url = 'https://dune-club.info/plugins/ext/data/om/index.php';
        $url = HD::append_dune_auth($url) . "&ext=.php";

        // TODO: revise. Now we cannot execute wget in caller context, so
        // just perform blocked request
//        return ActionFactory::wget($url, $str);

        HD::http_post_document($url, $str,
            array(CURLOPT_TIMEOUT => 3, CURLOPT_CONNECTTIMEOUT => 3));
        
        return null;
    }
}

?>
