<?php

class Playback
{
    private $cfg;
    private $stat;

    private $stat_movie = null;
    private $stat_torrent = null;

    private $episode_by_series_ndx = array();
    private $path_by_series_ndx = array();

    private $vod_id;
    private $switch_enabled = false;
    private $season = -1;
    private $episode = -1;
    private $start_stack = '';

    private $last_known_position = -1;

    private $started = false;
    private $start_tm;
    private $was_playing = false;
    private $was_buffering = false;
    private $osd_active = 0;

    private $debug_switched = false;

    // $last: not-null. Event record about last (current) playback state.
    // $last_arr: Array of several recent event records, including the last
    // (current) one.
    // Changed when handling playback_state_changed and playback_stop
    // events.
    private $last;
    private $last_arr = array();

    // Time to switch-track (try next torrent) if current state not changed
    private $state_limit_tm = 0;

    // Debug only - 'DEBUG FORCED SWITCH'
    private $forced_limit_tm = 0;

    // Last time 'seeking'/'initializing state changed to smth else
    private $last_end_seek_tm = 0;
    private $last_end_init_tm = 0;

    // Last time useraction (SwitchAudioTrack) occured
    private $last_useraction_tm = 0;

    private $recent_bufferings = array();

    public function __construct() {
        $this->cfg = MainHelper::$config;
        $this->stat = MainHelper::$pb_stat;

        $this->clear_last_on_stop(0);
    }

    private function ev_record($state, $tm, $limit_tm, $special_buffering) {
        return (object) array(
            'state' => $state, 'tm' => $tm, 'limit_tm' => $limit_tm,
            'special_buffering' => $special_buffering);
    }

    public static function my_print_r($v) {
        return preg_replace('|\s\s*|', ' ', print_r($v, 1));
    }

    private function push_last($evr) {
        $this->last = $evr;
        $this->last_arr[] = $evr;
        while (count($this->last_arr) > 3)
            array_shift($this->last_arr);
    }

    private function get_prev_last() {
        $len = count($this->last_arr);
        return $len < 2 ? null : $this->last_arr[$len - 2];
    }

    private function clear_last_on_stop($tm) {
        $this->last_arr = array();
        $evr = $this->ev_record('stopped', $tm, 0, false);
        $this->push_last($evr);
    }

    public function starting_new_playback() {
        hd_print("DEBUG reset last_known_position");
        $this->last_known_position = -1;
    }

    public function set_episode_by_series_ndx($episode_by_series_ndx) {
        $this->episode_by_series_ndx = $episode_by_series_ndx;
    }

    public function set_path_by_series_ndx($path_by_series_ndx) {
        $this->path_by_series_ndx = $path_by_series_ndx;
    }

    public function set_playback_parameters(
        $vod_id, $stack, $switch_enabled, $s=-1, $e=-1)
    {
        hd_print("DEBUG set_playback_parameters vod_id $vod_id, ".
            "sw $switch_enabled, S $s, E $e, stack: $stack");
        $this->vod_id = $vod_id;
        $this->start_stack = $stack;
        $this->switch_enabled = (int) $switch_enabled;
        $this->season = $s;
        $this->episode = $e;
    }

    private function check_vod_id($user_input)
    {
        $vod_id = HD::get($user_input, 'plugin_vod_id');
        if (!$vod_id)
            return true;

        $p1 = substr($vod_id, 0, strrpos($vod_id, '.'));
        $p2 = substr($this->vod_id, 0, strrpos($this->vod_id, '.'));
        if ($p1 == $p2)
            return true;

        hd_print("Playback I: vod changed $p2 -> $p1");
        $this->starting_new_playback();
        return false;
    }

    public function set_initial_pos($initial_pos) {
        if ($initial_pos > 0 && $this->last_known_position < 0) {
            $this->last_known_position = $initial_pos;
            hd_print("DEBUG last_known_position(1): $this->last_known_position");
        }
    }

    public function get_season() {
        return $this->season;
    }

    public function get_episode() {
        return $this->episode;
    }

    public function assign_episode_by_series_ndx($series_ndx) {
        if ($this->episode <= 0)
            return false;

        $e = HD::get($this->episode_by_series_ndx, $series_ndx, -1);
        if ($e > 0 && $e != $this->episode) {
            hd_print("DEBUG playback: current episode is now: $e");
            $this->episode = $e;
            $this->last_known_position = -1;
            return true;
        }

        return false;
    }

    private function clear_limits() {
        $this->recent_bufferings = array();
        $this->state_limit_tm = 0;
        $this->forced_limit_tm = 0;
    }

    public function switch_canceled() {
        $this->clear_limits();
    }

    public function set_state_limit($ev, $tm, $max_age) {
        $this->state_limit_tm = 0;
        if ($this->switch_enabled && $max_age > 0) {
            hd_print("Playback D: set max_age=$max_age for state '$ev->state'");
            $this->state_limit_tm = $tm + $max_age;
        }
    }

    private function is_fast_repeated_buffering($state, $tm)
    {
        if ($state != 'buffering' || $this->last->state != 'playing')
            return false;

        $prev_last = $this->get_prev_last();
        if (!$prev_last || $prev_last->state != $state)
            return false;
        if ($prev_last->special_buffering)
            return false;
        if ($tm < $prev_last->tm || $tm > $prev_last->tm + 1.0)
            return false;

        return true;
    }

    public function user_action(&$user_input)
    {
        $ev = HD::get($user_input, 'playback_event_type');
        if ($ev != 'SwitchAudioTrack')
            return null;

        $this->last_useraction_tm = microtime(true);
        $this->recent_bufferings = array();
        hd_print("Playback: user action detected");
        return null;
    }

    public function state_changed(&$user_input)
    {
        $pos = HD::get($user_input,'playback_position',-1);
        hd_print("DEBUG play state changed pos: $pos");

        $swd = $this->switch_enabled ?
            MainHelper::$switch_dialog : null;
        $tm = microtime(true);

        $vod_id_ok = $this->check_vod_id($user_input);

        if (isset($user_input->plugin_playback_start)) {
            if ($vod_id_ok && $pos > 0 && $this->last_known_position < 0)
            {
                $this->last_known_position = $pos;
                hd_print("DEBUG last_known_position(2): $pos");
            }
            $this->osd_active = 1;
            if (!$swd)
                return null;
            return MainHelper::$entry_handler->enter_osd_menu($user_input);
        }

        $this->osd_active = HD::get($user_input, 'osd_active', 0);

        if (isset($user_input->plugin_playback_stop)) {
            $this->clear_last_on_stop($tm);
            $this->started = false;
            $this->clear_limits();

            $duration = HD::get($user_input, 'plugin_vod_duration', 0);
            $is_error = HD::get($user_input, 'playback_error_code', 0) > 0;
            $is_eos = HD::get($user_input, 'playback_end_of_stream', 0);
            $stop_pressed = HD::get($user_input, 'playback_stop_pressed', 0);

            $stat_a = $this->stat->playback_stop($duration,
                $is_error, $is_eos, $stop_pressed);

            if ($swd && $is_error) {
                if (!$swd->opened) {
                    $start_action =
                        ActionFactory::action_from_user_input($user_input);
                    $a = $swd->open(true, $start_action);
                    return ActionFactory::composite(array($stat_a, $a));
                } else {
                    $swd->set_confirmed();
                }
            }

            if ($swd && $swd->opened)
            {
                $a = $swd->playback_stop($user_input);
                return ActionFactory::composite(array($stat_a, $a));
            }

            return $stat_a;
        }

        $res_action = null;

        $json = HD::get($user_input, 'plugin_playback_events');
        $obj = $json ? json_decode($json) : null;
        $events = HD::get($obj, 'events', array());
        foreach ($events as $ev) {
            $ev_tm = $ev->tm_sec + $ev->tm_usec / 1000000.0;
            $special_buffering = false;

            if ($ev->state != $this->last->state) {
                if ($this->state_limit_tm > 0 && $this->last->state != 'stopped') {
                    $tdiff = $tm - $this->last->tm;
                    $tstr = sprintf("%.2fs", $tdiff);
                    hd_print("Playback D: state '{$this->last->state}' changed in $tstr");
                }
                $this->state_limit_tm = 0;
                if ($this->last->state == 'seeking')
                    $this->last_end_seek_tm = $tm;
                if ($this->last->state == 'initializing')
                    $this->last_end_init_tm = $tm;

                $last_special = $this->last->special_buffering ||
                    $this->is_fast_repeated_buffering($ev->state, $ev_tm);

                $this->stat->state_changed($ev->state, $ev_tm,
                    $this->last->state, $this->last->tm, $last_special);
            }

            if ($ev->state == 'initializing') {
                $this->started = true;
                $this->start_tm = $ev_tm;
                $this->was_playing = false;
                $this->was_buffering = false;
                $this->clear_limits();
                $this->debug_switched = false;

                $series_ndx = HD::get($user_input, 'plugin_vod_series_ndx');
                if ($series_ndx >= 0)
                    $this->assign_episode_by_series_ndx($series_ndx);

                $this->init_stat();
                $this->update_stat_by_series_ndx($series_ndx);

                $this->set_state_limit($ev, $tm, 
                    $this->cfg->switch_long_start());

                $max_age = $this->cfg->debug_forced_switch_delay();
                if ($swd && $max_age > 0) {
                    $this->forced_limit_tm = $tm + $max_age;
                    hd_print("DEBUG FORCED SWITCH scheduled in $max_age sec");
                }
            } else if ($ev->state == 'playing') {
                if (!$this->was_playing)
                {
                    $this->stat->set_before_first_playing(
                        $ev_tm - $this->start_tm);
                }

                $this->was_playing = true;

                MainHelper::$torrent_history->dirty = true;

                if ($vod_id_ok && count($events) == 1 && $pos > 0) {
                    $this->last_known_position = $pos;
                    $this->stat->set_position($pos);
                    hd_print("DEBUG last_known_position(4): $pos");
                }
            } else if ($ev->state == 'seeking') {
                $this->set_state_limit($ev, $tm, 
                    $this->cfg->switch_long_seek());
                $this->recent_bufferings = array();
            } else if ($ev->state == 'buffering') {
                $check_needed = false;
                $special_buffering = true;
                $max_age = -1;
                if (!$this->was_buffering && !$this->was_playing) {
                    $max_age = $this->cfg->switch_long_start_buffering();
                    if ($swd)
                        hd_print("Playback: first buffering (lim $max_age)");
                }
                else if ($this->last_end_init_tm > 0 &&
                    $tm < $this->last_end_init_tm + 10.0)
                {
                    // If buffering started <5s after init ended =>
                    // considering buffering as part of init.
                    $this->was_playing = false;
                    $this->stat->set_before_first_playing(-1);
                    $max_age = $this->cfg->switch_long_start_buffering();
                    if ($swd)
                        hd_print("Playback: init buffering (lim $max_age)");
                }
                else if ($this->last_end_seek_tm > 0 &&
                    $tm < $this->last_end_seek_tm + 10.0)
                {
                    // If buffering started <5s after seek ended =>
                    // considering buffering as part of seek.
                    $max_age = $this->cfg->switch_long_seek_buffering();
                    if ($swd)
                        hd_print("Playback: seek buffering (lim $max_age)");
                }
                else if ($this->last_useraction_tm > 0 &&
                    $tm < $this->last_useraction_tm + 10.0)
                {
                    // NOTE: now switch audio tracks is handled similar to
                    // switch
                    $max_age = $this->cfg->switch_long_seek_buffering();
                    if ($swd)
                        hd_print("Playback: user-action buffering (lim $max_age)");
                }
                else if ($this->is_fast_repeated_buffering($ev->state, $ev_tm))
                {
                    $special_buffering = false;
                    $max_age = -1;

                    $prev_last = $this->get_prev_last();
                    $this->state_limit_tm = $prev_last->limit_tm;
                    $tdiff = sprintf("%.2fs", $ev_tm - $prev_last->tm);
                    $ev_tm = $prev_last->tm;

                    hd_print("Playback: glued bufferings (prev: -$tdiff)");
                }
                else
                {
                    $max_age = $this->cfg->switch_long_buffering();
                    $special_buffering = false;

                    if ($this->last->state == 'playing') {
                        $this->recent_bufferings[] = $ev_tm;
                        $check_needed = true;
                    }
                }

                if ($max_age != -1)
                    $this->set_state_limit($ev, $tm, $max_age);

                $this->was_buffering = true;

                if ($check_needed)
                {
                    $max_count = $this->cfg->switch_frequent_buffering_count();
                    while (count($this->recent_bufferings) > $max_count)
                        array_shift($this->recent_bufferings);

                    if ($swd && count($this->recent_bufferings) == $max_count)
                    {
                        $period = $this->cfg->switch_frequent_buffering_period();
                        $old_ev_tm = $this->recent_bufferings[0];
                        $tm_diff = $ev_tm - $old_ev_tm;
                        if ($tm_diff <= $period) {
                            $tm_str = sprintf("%.2fs", $tm_diff);
                            hd_print("Playback: TOO FREQUENT BUFFERING ($max_count for $tm_str)");
                            if ($this->osd_active && !$swd->opened && !$res_action) {
                                $this->stat->set_switch_reason('auto_switch');
                                $res_action = $swd->open();
                            }
                        }
                    }
                }
            }

            $evr = $this->ev_record($ev->state,
                $ev_tm, $this->state_limit_tm, $special_buffering);
            $this->push_last($evr);
        }

        return $res_action;
    }

    public function get_actions() {
        $eh = MainHelper::$entry_handler;
        $arr = array(
            GUI_EVENT_PLAYBACK_STATE_CHANGED =>
                $eh->custom_action('playback_state_changed'),
            GUI_EVENT_TIMER =>
                $eh->custom_action('playback_timer'),
        );
        if (defined('GUI_EVENT_PLAYBACK_USER_ACTION')) {
            $arr[GUI_EVENT_PLAYBACK_USER_ACTION] =
                $eh->custom_action('playback_user_action');
        }
        return $arr;
    }

    public function get_timer() {
        return ActionFactory::timer(2000, false, true);
    }

    private function change_behaviour($post_action=null) {
        return ActionFactory::change_behaviour(
            $this->get_actions(), $this->get_timer(), $post_action);
    }

    /**
     * @return false to cancel timer, true to continue timer, some action
     * to run action and continue timer.
     */
    private function perform_work(&$user_input)
    {
        if (!$this->started)
            return true;

        if (!$this->osd_active)
            return true;

        $swd = MainHelper::$switch_dialog;
        $tm = microtime(true);

        if ($this->state_limit_tm > 0 && $tm > $this->state_limit_tm)
        {
            $tdiff = $tm - $this->last->tm;
            $str = strtoupper($this->last->state);
            $tstr = sprintf("%.2fs", $tdiff);
            hd_print("Playback: TOO LONG $str: $tstr");
            $this->state_limit_tm = 0;
            if ($swd->opened) // TODO: revise
                return true;

            $this->stat->set_switch_reason('auto_switch');
            return $swd->open();
        }
        else if (!$this->debug_switched &&
            $this->forced_limit_tm > 0 && $tm > $this->forced_limit_tm)
        {
            $this->debug_switched = 1;
            hd_print("Playback: DEBUG FORCED SWITCH ACTIVATED");
            if ($swd->opened) // TODO revise
                return true;

            $this->stat->set_switch_reason('auto_switch');
            return $swd->open();
        }

        return true;
    }

    public function get_last_playing_position() {
        return $this->last_known_position;
    }

    public function is_started() {
        return $this->started;
    }

    public function timer(&$user_input)
    {
        $vod_id_ok = $this->check_vod_id($user_input);

        $pos = HD::get($user_input, 'playback_position', -1);
        if ($vod_id_ok && $this->started && $this->was_playing &&
            $this->last->state == 'playing' && $pos > 0)
        {
            hd_print("DEBUG last_known_position(3): $this->last_known_position");
            $this->last_known_position = $pos;
            $this->stat->set_position($pos);
        }

        if ($this->switch_enabled) {
            $res = $this->perform_work($user_input);
            if ($res === false)
                return null;
        } else {
            $res = null;
        }

        $action = $res === true ? null : $res;
        return $this->change_behaviour($action);
    }

    public function get_vod_stream_url($playback_url)
    {
        hd_print("Performing get_vod_stream_url: $playback_url");

        $action = MainHelper::$entry_handler->custom_action(
            'choose_playback_stream',
            array('playback_url' => $playback_url));

        throw new DuneException("Choose stream", 0, $action);
    }

    public function choose_playback_stream($user_input)
    {
        $server_checked = HD::get($user_input, 'server_checked');
        if (!$server_checked) {
            $cb = ActionFactory::action_from_user_input(
                $user_input, array('server_checked' => 1));
            return MainHelper::$entry_handler->ensure_own_torrserver(
                $user_input, $cb);
        }

        list($e, $torrent_id) = explode(':', $user_input->playback_url, 2);

        $st = MainHelper::$state;
        $mts = $st->play_mts;

        if ($e != $this->episode)
        {
            hd_print("DEBUG choose_playback_stream: E $this->episode -> $e");
            $this->episode = $e;

            $mts->set_episode($e);

            if ($mts->get_torrent_index($torrent_id) < 0)
            {
                hd_print("DEBUG choose_playback_stream: ".
                    "episode $e missing in current torrent");
                $t = reset($mts->get_torrents());
                $torrent_id = $t->id;
                // TODO: here we switch to torrent with best quality; maybe
                // we should try choose torrent with similar quality.

                hd_print("DEBUG choose_playback_stream ".
                    "new torrent: $t->name");
            }
        }

        $start_action = MainHelper::$progress->custom_action(
            'switch_open_torrent',
            array('e' => $e, 'torrent_id' => $torrent_id));

        $swd = MainHelper::$switch_dialog;
        return $swd->open(false, $start_action);
    }

    ///////////////////////////////////////////////////////////////////////

    public function set_stat_movie($m_id, $m_title, $m_year)
    {
        $this->stat_movie = (object) array(
            'id' => $m_id, 'title' => $m_title, 'year' => $m_year);
    }

    public function set_stat_torrent($t_id,
        $t_name, $t_q, $t_size_mb, $t_S, $t_L)
    {
        $this->stat_torrent = (object) array(
            'id' => $t_id, 'name' => $t_name, 'q' => $t_q,
            'size_mb' => $t_size_mb, 'S' => $t_S, 'L' => $t_L);
    }

    public function init_stat()
    {
        $m = $this->stat_movie;
        $t = $this->stat_torrent;

        $this->stat->set_movie(
            $m->id, $m->title, $m->year,
            $this->season, $this->episode);

        $this->stat->cleanup();

        $this->stat->set_torrent($t->id,
            $t->name, $t->q, $t->size_mb, $t->S, $t->L);
    }

    public function update_stat_by_series_ndx($series_ndx)
    {
        $path = HD::get($this->path_by_series_ndx, $series_ndx, '');
        $this->stat->t_file = $path;

        $this->stat->episode = $this->episode;
    }
}

?>
