<?php

class Settings
{
    public function is_setting_name($name)
    {
        return in_array($name,
            $this->get_all_names());
    }

    public function get_all_names()
    {
        return array(
            'sort_video_streams',
            'max_quality',
            'min_quality',
            'min_availability',
            'ext_torrserver',
        );
    }

    public function get_def_value($name)
    {
        switch ($name) {
            case 'sort_video_streams':
                return 'auto';
            case 'max_quality':
                return 'auto';
            case 'min_quality':
                return 'any';
            case 'min_availability':
                return '0';
            case 'ext_torrserver':
                return '';
            default:
                return '';
        }
    }

    public function get_ui_label($name)
    {
        switch ($name) {
            case 'sort_video_streams':
                return T::t('label_sort_video_streams');
            case 'max_quality':
                return T::t('label_max_quality');
            case 'min_quality':
                return T::t('label_min_quality');
            case 'min_availability':
                return T::t('label_min_availability');
            case 'ext_torrserver':
                return T::t('label_ext_torrserver');
            default:
                return '';
        }
    }

    public function get_ui_opts($name)
    {
        switch ($name) {
            case 'sort_video_streams':
                return array(
                    'auto' => T::g('auto_yes_no_choice_auto'),
                    'quality' => T::t('sort_quality'),
                    'availability' => T::t('sort_availability'));
            case 'max_quality':
                return array(
                    'auto' => T::g('auto_yes_no_choice_auto'),
                    '4k' => '4K',
                    'fullhd' => 'Full HD',
                    'hd' => 'HD',
                );
            case 'min_quality':
                return array(
                    '4k' => '4K',
                    'fullhd' => 'Full HD',
                    'hd' => 'HD',
                    'any' => T::t('quality_any'),
                );
            case 'min_availability':
                return array(
                    '0' => '0',
                    '1' => '1',
                    '2' => '2',
                    '3' => '3',
                    '5' => '5',
                    '10' => '10',
                    '20' => '20',
                    '30' => '30',
                    '50' => '50',
                    '100' => '100',
                );
            case 'ext_torrserver':
                return 'text';
            default:
                return '';
        }
    }

    public function get_value($name)
    {
        $plugin_cookies = MainHelper::$plugin->get_plugin_cookies();
        return HD::get($plugin_cookies,
            $name, $this->get_def_value($name));
    }

    public function get_value_caption($name)
    {
        $v = $this->get_value($name);
        $opts = $this->get_ui_opts($name);
        if (is_array($opts))
            return HD::get($opts, $v, '');
        return $v;
    }

    public function set_value($name, $value)
    {
        $old_value = $this->get_value($name);
        if ($value == $old_value)
            return false;

        hd_print("Set($name): $old_value -> $value");
        $plugin_cookies = MainHelper::$plugin->get_plugin_cookies();
        $plugin_cookies->$name = $value;

        MainHelper::$state->dirty = true;

        $this->do_apply();

        return true;
    }
    
    public function toggle_value($name)
    {
        $opts = $this->get_ui_opts($name);
        if (!is_array($opts))
            return;

        $v = $this->get_value($name);
        $values = array_keys($opts);
        $ndx = array_search($v, $values);
        if (false === $ndx)
            return;

        $ndx = ($ndx + 1) % count($values);
        $v2 = $values[$ndx];
        $this->set_value($name, $v2);
    }

    ///////////////////////////////////////////////////////////////////////

    public $dirty = true;
    public $min_q = 0;
    public $max_q = TorrentInfo::QRES_2160P;
    public $min_s = 0;

    public $tv_4k = -1;

    public $ext_torrserver = '';

    public function load_tv_info()
    {
        $path = "/tmp/run/tv_4k.txt";
        if (is_file($path))
        {
            $this->tv_4k = (int) file_get_contents($path);
            $this->dirty = true;
        }
    }

    public function apply_if_dirty()
    {
        if (!$this->dirty)
            return;
        $this->do_apply();
    }

    private function do_apply()
    {
        $this->dirty = false;
        $this->min_s = (int) $this->get_value('min_availability');
        $this->min_q = $this->min_quality();
        $this->max_q = $this->max_quality();
        $this->ext_torrserver = $this->make_ext_torrserver();
        hd_print("DEBUG apply: min_s: $this->min_s, q: $this->min_q..$this->max_q");
        hd_print("DEBUG apply: ext_torrserver = $this->ext_torrserver");

        TorrServer::unsetIp();
    }

    private function max_quality()
    {
        switch ($this->get_value('max_quality'))
        {
            case 'auto':
                return $this->tv_4k == 0 ?
                    TorrentInfo::QRES_1080P :
                    TorrentInfo::QRES_2160P;
            case '4k':
                return TorrentInfo::QRES_2160P;
            case 'fullhd':
                return TorrentInfo::QRES_1080P;
            case 'hd':
                return TorrentInfo::QRES_720P;
            default:
                return TorrentInfo::QRES_1080P;
        }
    }

    private function min_quality()
    {
        switch ($this->get_value('min_quality'))
        {
            case '4k':
                return TorrentInfo::QRES_2160P;
            case 'fullhd':
                return TorrentInfo::QRES_1080P;
            case 'hd':
                return TorrentInfo::QRES_720P;
            case 'any':
            default:
                return 0;
        }
    }

    private function make_ext_torrserver()
    {
        $v = $this->get_value('ext_torrserver');
        if (!$v)
            return '';
        if (false === strpos($v, ':'))
            return "$v:8090";
        return $v;
    }
}

?>
