<?php

require_once 'tordb/TorDB.php';

class JacketDB extends TorDB
{
    const CONFIG_URL =
        'http://dune-club.info/plugins/ext/plugins/dunemdb_supplier_online_movies/config.txt';
    const DEF_QUERY_FMT =
        'https://s1.dune-club.info/plugins/dunemdb_supplier_online_movies/q?q=%s';
    const CONFIG_EXPIRE_TM = 7200;

    private function config_path() {
        return DuneSystem::$properties['tmp_dir_path'] .
            "/config.txt";
    }
    private function last_check_config_tm_path() {
        return DuneSystem::$properties['tmp_dir_path'] .
            "/last_check_config_tm.txt";
    }

    private function config_doc_to_query_fmt($doc) {
        $props = array();
        Properties::do_parse_string($props, $doc);
        return HD::get($props, 'query_fmt');
    }

    private $query_fmt = null;

    private static function query_fmt_is_valid($fmt)
    {
        if (!$fmt ||
            (0 !== strpos($fmt, 'http://') &&
             0 !== strpos($fmt, 'https://')))
        {
            return false;
        }

        $pos = strpos($fmt, '%s');
        if (!$pos)
            return false;
        $pos = strpos($fmt, '%s', $pos + 2);
        if ($pos)
            return false;
        return true;
    }

    private function get_query_fmt()
    {
        if (is_file("/config/online_movies_jacket.txt"))
            return file_get_contents("/config/online_movies_jacket.txt");

        $cfg_path = $this->config_path();

        if (!isset($this->query_fmt))
        {
            $doc = is_file($cfg_path) ? file_get_contents($cfg_path) : null;
            $query_fmt = $this->config_doc_to_query_fmt($doc);
            $this->query_fmt = $query_fmt ? $query_fmt : self::DEF_QUERY_FMT;

            if ($this->query_fmt == self::DEF_QUERY_FMT)
                hd_print("Jacket D: query is default: $query_fmt");
            else
                hd_print("Jacket D: cached query is $query_fmt");
        }

        $tm_path = $this->last_check_config_tm_path();
        $last_tm = is_file($tm_path) ? (int) file_get_contents($tm_path) : 0;
        $cur_tm = time(0);
        $tdiff = abs($cur_tm - $last_tm);
        if ($this->query_fmt && $tdiff < self::CONFIG_EXPIRE_TM)
        {
            $tm_left = $cur_tm + self::CONFIG_EXPIRE_TM - $last_tm;
            hd_print("Jacket D: next query req in ${tm_left}s");
            return $this->query_fmt;
        }

        $opts = array(CURLOPT_TIMEOUT => 10, CURLOPT_CONNECTTIMEOUT => 10);
        $url = self::CONFIG_URL;
        $doc = HD::http_get_document($url, $opts);
        $query_fmt = $this->config_doc_to_query_fmt($doc);

        $query_valid = self::query_fmt_is_valid($query_fmt);

        $last_tm = $query_valid ? $cur_tm :
            $cur_tm - self::CONFIG_EXPIRE_TM + 60;

        if ($query_valid && $query_fmt != $this->query_fmt)
        {
            hd_print("Jacket I: got new query: $query_fmt");
            $this->query_fmt = $query_fmt;
            file_put_contents($cfg_path, $doc);
        }
        else if ($query_valid)
            hd_print("Jacket I: got same query: $query_fmt");
        else
            hd_print("Jacket I: got invalid query: $query_fmt");

        file_put_contents($tm_path, $last_tm);
        $tm_left = time(0) + self::CONFIG_EXPIRE_TM - $last_tm;
        hd_print("Jacket D: next query req in ${tm_left}s");
        return $this->query_fmt;
    }

    private static function unit_to_text($v, $name)
    {
        $text = $v >= 100 ? round($v) :
            ($v >= 10 ? sprintf("%.1f", $v) : sprintf("%.2f", $v));
        return "$text $name";
    }

    private static function mb_to_text($mb)
    {
        if ($mb >= 1024)
            return self::unit_to_text($mb / 1024.0, 'GB');
        return self::unit_to_text($mb, 'MB');
    }

    private function convert_size_to_string($json)
    {
        return preg_replace('|"Size" *: *(\d\d*)|', '"Size":"$1"', $json);
    }

    public function request_torrents(
        $title, $year, &$idset, $dump_fp, $ndx,
        $lang, $region,
        &$all_torrents, &$good_ids)
    {
        // Some fix on title, API failed on queries like "D'Artagnan".
        $title = str_replace("'", " ", $title);

        $query = $title;
        if ($year && $year > 1900)
            $query = "$title $year";
        $equery = urlencode($query);

        $opts = array(
            CURLOPT_TIMEOUT => 120,
            CURLOPT_CONNECTTIMEOUT => 120);

        $fmt = $this->get_query_fmt();
        $url = sprintf($fmt, $equery);
        $url = HD::append_dune_auth($url);
        $url = $url . '&lang=' . $lang . '&region=' . $region;

        $ts = microtime(1);
        $content = HD::http_get_document($url, $opts);
        $t_str = sprintf("%.2fs", microtime(1) - $ts);
        $obj = $content ?
            json_decode($this->convert_size_to_string($content)) :
            null;
        $results = HD::get($obj, 'Results');
        if (!$results)
        {
            $n = $content ? strlen($content) : 0;
            if ($n < 200)
                hd_print("Jacket I: no torrents in response (took $t_str) ($n bytes): $content");
            else
            {
                hd_print("Jacket I: no torrents in response (took $t_str) ($n bytes)");
                if ($n > 2048)
                    $content = substr($content, 2048) . "...";
                hd_print("Jacket D: response: $content");
            }
            return $ndx;
        }

        $num_dups = 0;

        foreach ($results as $res)
        {
            $id = HD::get($res, 'MagnetUri', HD::get($res, 'Link'));
            if (!$id)
            {
                hd_print("JacketDB W: ID missing");
                continue;
            }

            $title = HD::get($res, 'Title');
            if (!$title)
            {
                hd_print("JacketDB W: Title missing");
                continue;
            }

            $source = HD::get($res, 'Tracker',
                HD::get($res, 'TrackerId'));

            $S = HD::get($res, 'Seeders');
            $L = HD::get($res, 'Peers');

            $size_b = HD::get($res, 'Size', 0);

            $uid = HD::get($res, 'Guid');
            if (!$uid)
                $uid = md5($title) . "." . md5($source);

            $id2 = "$source:$S:$L:$size_b:$title";
            if (isset($idset[$id2])) {
                $num_dups++;
                continue;
            }
            $idset[$id2] = 1;

            $size_mb = (float) $size_b / 1048576.0;
            $size = self::mb_to_text($size_mb);

            $group_id = TorrentHelper::get_group_id($id);

            $t = (object) array(
                'id' => $id,
                'group_id' => $group_id,
                'name' => $title,
                'S' => $S,
                'L' => $L,
                'size' => $size,
                'size_mb' => $size_mb,
                'uid' => $uid,
            );

            $t->extra_item = $size . ' (' . $S . '/' . $L . ')';

            $t->info = '||' . $title . 
                '|Размер: '. $size . 
                '||S: '. $S . ' L: '. $L .
                '||' . $source;

            // For debug.
            $t->query = $query;

            $all_torrents[] = $t;

            $good = 1;
            $good_ids[$t->id] = 1;

            if ($dump_fp) {
                $arr = array(
                    "L$ndx", 'OK',
                    $t->size, $t->S, $t->L, $t->name);
                fwrite($dump_fp, implode(',', $arr) . "\n");
            }
            $ndx++;
        }

        $total = count($results);
        $perc = sprintf("%.2f", $num_dups * 100.0 / $total);
        hd_print("$total torrents loaded (took $t_str), dups $num_dups($perc%)");

        return $ndx;
    }

    private static function parse_location($content)
    {
        if (!$content)
            return null;

        $headers = explode("\n", $content);
        foreach ($headers as $h)
        {
            $h = trim($h);
            if (0 !== stripos($h, 'location:'))
                continue;

            return trim(substr($h, 9));
        }
    }

    public function get_torrent_file($torrent_id)
    {
        if (0 === strpos($torrent_id, "magnet:"))
            return array('status' => true, 'path' => $torrent_id);

        if (0 === strpos($torrent_id, "http"))
        {
            return array('status' => true, 'path' => $torrent_id);
            /*
            $opts = array(
                CURLOPT_FOLLOWLOCATION => false,
                CURLOPT_HEADER => true);
            $content = HD::http_get_document($torrent_id, $opts);
            $magnet = self::parse_location($content);
            if ($magnet)
            {
                hd_print("Got magnet from magnet: $magnet");
                return array('status' => true, 'path' => $magnet);
            }
            return array("status" => false,"descr" => "Возможно произошла ошибка при поиске магнет ссылки");
            */
        }

        return array("status" => false,"descr" => "Неизвестный формат ссылки");
    }

    public function support_search_with_year()
    {
        return false;
    }
}

?>
