<?php

require_once 'tordb/TorDB.php';

class TorLookDB extends TorDB
{
    public function request_torrents(
        $title, $year, &$idset, $dump_fp, $ndx,
        $lang, $region,
        &$all_torrents, &$good_ids)
    {
        $tor_movie = (object) array(
            'name' => $title,
            'name_original' => $title,
            'year' => $year);

        $ts = microtime(true);
        $cfg = MainHelper::$config;

        $feed = '1';
        $items = TorLook_Class::getShortTorrentRange($tor_movie, null, $feed);

        $tdiff = microtime(true) - $ts;
        $ecode = TorLook_Class::$lastCurlExitCode;
        if (!$items && $ecode == 28 && $tdiff > 15.0 &&
            !$cfg->use_orig_torlook_addr &&
            Config::$temp_force_orig_torlook_addr == -1)
        {
            hd_print("DEBUG retrying with original torlook addr");

            Config::$temp_force_orig_torlook_addr = 1;
            $cfg->reload();

            $feed = '1';
            $items = TorLook_Class::getShortTorrentRange($tor_movie, null, $feed);

            if (!$items) {
                Config::$temp_force_orig_torlook_addr = 0;
                $cfg->reload();
            }
        }

        if (!$items)
            return $ndx;

        $num_dups = 0;

        $prev_S = HD::INT_MAX;
        $good = true;
        foreach ($items as $t) {
            // Keep priority for first range of results: it is ordered by
            // decreasing 'S' (seeds) value. Presumably, first range
            // contains movies with sutable audio tracks for current
            // langugage.
            if ($good) {
                if ($t->S > $prev_S) {
                    hd_print("DEBUG good range ended on: $t->name (S$t->S after $prev_S)");
                    $good = false;
                } else {
                    $prev_S = $t->S;
                }
            }

            if (isset($idset[$t->id2])) {
                $num_dups++;
                continue;
            }
            $idset[$t->id2] = 1;
            $all_torrents[] = $t;
            if ($good)
                $good_ids[$t->id] = 1;

            if ($dump_fp) {
                $arr = array(
                    "L$ndx",
                    $good ? 'OK' : 'BAD',
                    $t->size, $t->S, $t->L, $t->name);
                fwrite($dump_fp, implode(',', $arr) . "\n");
            }
            $ndx++;
        }

        if ($num_dups > 0)
        {
            $total = count($items);
            $perc = sprintf("%.2f", $num_dups * 100.0 / $total);
            hd_print("Torrent dups removed: $num_dups of total $total ($perc%)");
        }

        return $ndx;
    }

    public function get_torrent_file($torrent_id)
    {
        return TorLook_Class::getTorrentFile($torrent_id);
    }

    public function support_search_with_year()
    {
        return true;
    }
}

?>
