<?php

class TorrentHelper
{
    const ID = 'torrent_helper';

    // For debug.
    const DEBUG_ADD_NAME_TO_TAGS = 1;

    const ERR_TORRENT_NOT_STARTED = 1;
    const ERR_TORRENT_RESOLVE_FAILED = 2;
    const ERR_TORRENT_ADD_FAILED = 3;

    private static $vod_id_counter = 0;

    ///////////////////////////////////////////////////////////////////////

    public static function get_group_id($torrent_id)
    {
        return str_replace('/', '_', $torrent_id);
    }

    public static function get_torrent_item_media_url($s, $torrent_id)
    {
        return json_encode(
            array('season' => (int) $s, 'torrent_id' => $torrent_id));
    }

    public static function open_torrent(
        $switch_mode, $switch_position,
        $movie_id, $season, $torrent_id, $torrent_uid, $stack,
        $chosen_file_id, $chosen_file_pos, $chosen_op)
    {
        if (!TorrServer::getOnlineStatusServer()) {
            hd_print("Error: TorrServer not started, ip: " . TorrServer::getIp());
            return self::ERR_TORRENT_NOT_STARTED;
        }

        $torfile = MainHelper::$tordb->get_torrent_file($torrent_id);
        if (!$torfile || !$torfile['status']) {
            hd_print("Error: failed to retrieve magnet link");
            return self::ERR_TORRENT_RESOLVE_FAILED;
        }

        $torpath = $torfile['path'];

        // torrServAction 1: don't save to DB, 2: save to DB.
        $hash = TorrServer::torrserverAddTorrent($torpath, 1);
        if (!$hash)
        {
            hd_print("Error: failed to add torrent to server");
            return self::ERR_TORRENT_ADD_FAILED;
        }

        hd_print("DEBUG torrent added with hash $hash: $torpath");

        return MainHelper::$add_torrent_dialog->run(
            $switch_mode, $switch_position,
            $movie_id, $season, $torrent_id, $torrent_uid, $hash, $stack,
            $chosen_file_id, $chosen_file_pos, $chosen_op);

/*
        if ($switch_mode)
            return $add_action;

        // TODO: revise whether torrent_list request is really needed
        $ctx = TorrServer::getTorrentListRequestCtx();
        $wget_action = ActionFactory::wget(
            $ctx->url, $ctx->post_data,
            array(CURLOPT_TIMEOUT => $ctx->timeout),
            'omov_torlist', null, 'torlist_',
            $eh->custom_action('apply_torlist'));
        $actions = array($wget_action, $add_action);
        // !!! return ActionFactory::composite($actions);
*/
    }

    public static function open_torrent_error_action($res, $stop_playback)
    {
        if ($res === TorrentHelper::ERR_TORRENT_NOT_STARTED) {
            return ActionFactory::show_error(false,
                T::t('errmsg_online_movies_not_available'),
                array(T::g1('dialog_system_error__1', "02")),
                $stop_playback);
        } else if ($res === TorrentHelper::ERR_TORRENT_RESOLVE_FAILED) {
            return ActionFactory::show_error(false,
                T::t('errmsg_connect_to_source_failed'),
                array(T::g1('dialog_system_error__1', "03")),
                $stop_playback);
        } else if ($res === TorrentHelper::ERR_TORRENT_ADD_FAILED) {
            return ActionFactory::show_error(false,
                T::t('errmsg_connect_to_source_failed'),
                array(T::g1('dialog_system_error__1', "04")),
                $stop_playback);
        }
        return null;
    }

    public static function menu_playback_finish($user_input,
        $post_action=null)
    {
        $urls = UrlHelper::get_urls_stack($user_input);

hd_print("DEBUG urls to invalidate: ".print_r($urls,1));

        $a = ActionFactory::invalidate_folders($urls, $post_action);
        return MainHelper::$entry_handler->update_wh($a, true, true);
    }

    public static function get_vod_movie_base(
        $torrent_id, $torrent_uid, $initial_position, $autostart)
    {
        $st = MainHelper::$state;
        $mts = $st->play_mts;
        $m = $mts->get_movie();

        self::$vod_id_counter++;
        $vod_id = $m->id . '.' . self::$vod_id_counter;
        $vod_movie = new Movie($vod_id);

        $caption = $m->title;

        $inf = $mts->get_torrent_info($torrent_id);
        $t = $mts->get_torrent_by_id($torrent_id);

        $pb = MainHelper::$playback;
        $title = HD::get($m, 'native_title');
        if (!$title)
            $title = $m->title;
        $pb->set_stat_movie(
            $m->id, $title, $m->year);
        $pb->set_stat_torrent(
            $t->id, $t->name, $inf->quality_str,
            $inf->size_mb, $t->S, $t->L);

        $sub_name = $inf->get_caption(-1, false);
        $extra_text = HD::get($t, 'extra_item');
        if ($extra_text)
            $sub_name .= " | " . $extra_text;

        $desc = $m->description ?
            "$sub_name\n\n$m->description" : $sub_name;

        $vod_movie->set_data($caption, $m->native_title,
            $desc, $m->poster_url, -1, $m->year,
            '', '', '', '', '', '', '', '', '');

        $vod_movie->initial_position = $initial_position;
        hd_print("DEBUG vod_play initial_position: $initial_position");

        $vod_movie->recent_plugin = 'shell_ext';
        $vod_movie->recent_action_params = array(
            'handler_id' => 'entry',
            'control_id' => 'play_recent',
            'sup_id' => 'online_movies',
            'movie_id' => $m->id,
            'sup_data' => (int) $autostart);

        $th = MainHelper::$torrent_history;
        $tag = $th->make_torrent_tag($torrent_uid);
        $vod_movie->recent_tags[] = $tag;
        if (self::DEBUG_ADD_NAME_TO_TAGS)
            $vod_movie->recent_tags[] = "debug_online_movies:$sub_name";

        $pb = MainHelper::$playback;
        $vod_movie->set_behaviour($pb->get_actions(), $pb->get_timer());

        return $vod_movie;
    }

    public static function check_torrent_item_suitable_for_autoswitch_playback(
        $series, $sel_id)
    {
        $cfg = MainHelper::$config;
        if (!$cfg->autoswitch_enabled())
            return false;

        $st = MainHelper::$state;
        $mts = $st->play_mts;
        $m = $mts->get_movie();

        if (!$m->series_type)
            return count($series) == 1;

        $sel_ser = null;
        if ($sel_id) {
            foreach ($series as $ser) {
                if ($ser['id'] == $sel_id) {
                    $sel_ser = $ser;
                    break;
                }
            }
        }
        if (!$sel_ser) {
            if ($sel_id)
                hd_print("Playback W: series not found by id: $sel_id");
            $sel_ser = reset($series);
        }

        $e = HD::get($sel_ser, 'e', -1);
        $s = HD::get($sel_ser, 's', -1);
        $fname = $sel_ser['title'];
        if ($e >= 1 && $s >= 1 && $s == $mts->get_season())
        {
            hd_print("Playback of S$s, start from E$e (switch: on)");
            return (object) array('s' => $s, 'e' => $e);
        }

        return false;
    }

    /**
     * Return vod_movie for season, containing all episodes, known by mdb.
     * If given torrent is opened (@series is nonempty), corresponding
     * items constructed using playback urls for current torrent; other
     * items have playback_url_as_stream_url=false, get_stream_url() from
     * shell FW launches switch action.
     */
    public static function get_all_episodes_vod_movie(
        $stack, $autostart, $torrent_id, $torrent_uid, $sel_e,
        $initial_position=-1, $series=null)
    {
        $st = MainHelper::$state;
        $mts = $st->play_mts;
        $m = $mts->get_movie();
        $season = $mts->get_season();

        hd_print("DEBUG get_all_episodes_vod_movie(): ".
            "stack $stack, autostart $autostart, inipos $initial_position");
        hd_print("DEBUG get_all_episodes_vod_movie(): torrent_uid $torrent_uid");
        hd_print("DEBUG get_all_episodes_vod_movie(): torrent_id $torrent_id");

        $vod_movie = TorrentHelper::get_vod_movie_base(
            $torrent_id, $torrent_uid, $initial_position, $autostart);

        $vod_movie->id = "E$sel_e.$vod_movie->id";

        $ser_by_e = array();
        $path_by_e = array();
        if ($m->series_type && $series) {
            foreach ($series as $path => $ser) {
                $e = HD::get($ser, 'e', -1);
                $s = HD::get($ser, 's', -1);
                if ($s == $season && $e >= 0 && !isset($ser_by_e[$e]))
                {
                    $ser_by_e[$e] = $ser;
                    $path_by_e[$e] = $path;
                }
            }
        }

        $pb = MainHelper::$playback;
        $pb->set_playback_parameters($vod_movie->id, $stack, 1, $season, $sel_e);

        if ($season >= 1 && count($m->season_numbers) > 1) {
            $vod_movie->name = T::g2('shell_ext_plugin_item_movie_season__2',
                $vod_movie->name, $season);
        }

        $pub_ndx = $mts->get_torrent_public_index($torrent_id);
        $vod_movie->description = "#$pub_ndx. " . $vod_movie->description;

        $th = MainHelper::$torrent_history;

        $sns = HD::get($m->season_numbers, $season);
        $episodes_count = HD::get($sns, 'episodes_count', 0);

        $path_by_sndx = array();

        $e_by_sndx = array();
        for ($e = 1; $e <= $episodes_count; $e++)
        {
            $ser_title = T::g1('shell_ext_plugin_item_episode__1', $e);

            $ser = HD::get($ser_by_e, $e);
            if (!$ser)
            {
                if ($e == $sel_e)
                    hd_print("No stream yet provided for current episode");

                // TODO: use 'item_episode_with_name__2'
                $playback_url = "$e:$torrent_id";
                $vod_movie->add_series_data("E$e",
                    $ser_title, $playback_url, false);
                continue;
            }
            $fpath = HD::get($path_by_e, $e);

            $recent_uid = $th->make_file_uid(
                $m->id, $torrent_uid, $ser['id']);
            $e = HD::get($ser, 'e', -1);
            $s = HD::get($ser, 's', -1);

            $sndx = count($vod_movie->series_list);
            $e_by_sndx[$sndx] = $e;
            $path_by_sndx[$sndx] = $fpath;

            $pl_url = $ser['playback_url'];
            $pl_url .= "|||dune_params|||http_online_subs:1,file_id:$fpath";

            $mdb_info = ActionFactory::file_movie_info(
                $m->id, $m->title, $m->poster_url, $season, $e);
            $vod_movie->add_series_data("E$e",
                $ser_title, $pl_url, true,
                $recent_uid, $mdb_info);
        }

        if (!$m->series_type) {
            if ($series && count($series) == 1) {
                $keys = array_keys($series);
                $fpath = $keys[0];
                $path_by_sndx[0] = $fpath;

                $ser = reset($series);

                $pl_url = $ser['playback_url'];
                $pl_url .= "|||dune_params|||http_online_subs:1,file_id:$fpath";

                $recent_uid = $th->make_file_uid(
                    $m->id, $torrent_uid, $ser['id']);
                $mdb_info = ActionFactory::file_movie_info(
                    $m->id, $m->title, $m->poster_url);
                $vod_movie->add_series_data("single",
                    "", $pl_url, true,
                    $recent_uid, $mdb_info);

            } else {
                $playback_url = "-1:$torrent_id";
                $vod_movie->add_series_data("single",
                    "", $playback_url, false);
            }
        }

        $enter_menu_action =
            MainHelper::$entry_handler->custom_action('enter_osd_menu');
        $mitem = ActionFactory::get_osd_menu_item('video_streams',
            T::t('action_video_stream_list'), false);
        $vod_movie->set_extra_menu(
            array($mitem), $enter_menu_action);

        $pb->set_episode_by_series_ndx($e_by_sndx);
        $pb->set_path_by_series_ndx($path_by_sndx);

        return $vod_movie;
    }
}

?>
