<?php

require_once 'lib/user_input_handler_registry.php';
require_once 'lib/abstract_preloaded_regular_screen.php';

class TorrentsScreen extends AbstractPreloadedRegularScreen
    implements UserInputHandler
{
    const ID = 'torrents';

    const PAGE_SIZE = 9;

    const MOV_ICON_W = 52;
    const MOV_ICON_H = 70;
    const MOV_ICON_DX = 13;

    const OTHER_ICON_W = 48;
    const OTHER_ICON_H = 48;
    const OTHER_ICON_DX = 15;

    /**
     * @param season: -1 for single movie, 0 for 'others', >0 for
     * particular season.
     */
    public static function get_media_url_str(
        $movie_id, $season, $episode, $chosen_only, $stack)
    {
        $arr = array(
            'screen_id' => self::ID,
            'movie_id' => $movie_id,
            'season' => (int) $season,
            'episode' => (int) $episode,
            'chosen_only' => (bool) $chosen_only,
            'stack' => $stack,
        );
        return MediaURL::encode($arr);
    }

    ///////////////////////////////////////////////////////////////////////

    private $chosen_file_id = null;
    private $chosen_file_pos = null;
    private $chosen_op = null;
    private $debug = false;

    private $fast_timer_countdown = 0;

    public function __construct()
    {
        parent::__construct(self::ID);

        $this->debug = is_file("/config/omov_debug");

        $this->set_folder_view_index_attr_name('view_index');

        UserInputHandlerRegistry::get_instance()->register_handler($this);
    }

    public function get_handler_id()
    {
        return self::ID;
    }

    public function custom_action($control_id,
        $params=null, $timeout_ms=null, $show_dialog_delay_ms=null,
        $skip_on_play_restart=false)
    {
        $a = UserInputHandlerRegistry::create_action($this,
            $control_id, $params, $timeout_ms, $show_dialog_delay_ms);
        if ($skip_on_play_restart)
            $a[GuiAction::flags] = GUI_ACTION_FLAG_SKIP_ON_PLAY_RESTART;
        return $a;
    }

    public function get_view_index(&$plugin_cookies) {
        return HD::get($plugin_cookies, 'view_index', 0);
    }

    ///////////////////////////////////////////////////////////////////////

    public function get_folder_view(MediaURL $media_url, &$plugin_cookies)
    {
        return parent::get_folder_view($media_url, $plugin_cookies);
    }

    protected function do_get_folder_views(&$plugin_cookies)
    {
        $loading_url = "plugin_file://%shell_ext%/icons/movies_small.png";
        $load_failed_url = "plugin_file://%shell_ext%/icons/no_movie_poster.png";

        $height = 700;

        $view = array(
            PluginRegularFolderView::view_params => array_merge(
                array(
                    ViewParams::num_cols => 1,
                    ViewParams::num_rows => 11,
                    ViewParams::paint_details => false,

                    ViewParams::scroll_animation_enabled => true,

                    ViewParams::content_box_y => 250,
                    ViewParams::content_box_height => $height,
                )),

            PluginRegularFolderView::base_view_item_params => array(
                ViewItemParams::item_padding_top => 0,
                ViewItemParams::item_padding_bottom => 0,
                ViewItemParams::icon_valign => VALIGN_CENTER,
                ViewItemParams::item_layout => HALIGN_LEFT,
                ViewItemParams::item_caption_width => 1570,
                ViewItemParams::item_caption_font_size => FONT_SIZE_SMALL,
                ViewItemParams::icon_dx => self::OTHER_ICON_DX,
                ViewItemParams::icon_width => 38,
                ViewItemParams::icon_height => 38,
            ),

            PluginRegularFolderView::not_loaded_view_item_params => array
            (
                ViewItemParams::icon_path => $loading_url,
            ),

            PluginRegularFolderView::load_failed_view_item_params => array
            (
                ViewItemParams::icon_path => $load_failed_url,
                ViewItemParams::icon_width => self::MOV_ICON_W,
                ViewItemParams::icon_height => self::MOV_ICON_H,
                ViewItemParams::icon_dx => self::MOV_ICON_DX,
            ),
        );
        return array($view, $view);
    }

    ///////////////////////////////////////////////////////////////////////

    public function get_action_map(MediaURL $media_url, &$plugin_cookies)
    {
        $green_action = $this->custom_action('b_green');

        $ss = MainHelper::$settings;
        $caption = $ss->get_value_caption('sort_video_streams');
        $green_action[GuiAction::caption] = T::t1('action_sort__1', $caption);

        return array(
            GUI_EVENT_KEY_ENTER => $this->custom_action('enter'),
            GUI_EVENT_KEY_INFO => $this->custom_action('info'),
            GUI_EVENT_KEY_B_GREEN => $green_action,
            GUI_EVENT_KEY_POPUP_MENU => $this->custom_action('popup'),
            GUI_EVENT_MENU_PLAYBACK_FINISH =>
                $this->custom_action("menu_playback_finish",
                    null, null, null, true),
            GUI_EVENT_TIMER => $this->custom_action('timer'),
        );
    }

    public function get_timer($media_url)
    {
        $st = MainHelper::$state;
        if ($this->fast_timer_countdown > 0 || $st->installing_ts)
            return ActionFactory::timer(1000);
        return ActionFactory::timer(5000);
    }

    ///////////////////////////////////////////////////////////////////////

    private function is_collapsed_torrent_id($mts,
        $season, $episode, $torrent_id)
    {
        $torrents = $mts->get_torrents($season, $episode);

        $group_ndx = -1;
        $group_size = 0;
        $last_q = -1;

        $num_added = 0;
        $bad_met = false;
        foreach ($torrents as $k => $t)
        {
            $inf = $mts->get_torrent_info($t->id);
            $good = $mts->is_good_torrent($t->id);

            if (!$bad_met && !$good) {
                $bad_met = true;
                if ($k > 0)
                    break;
            }

            $q = $inf->quality;
            if ($q != $last_q) {
                $last_q = $q;
                $group_ndx++;
                $group_size = 0;
            }

            $max_group_size = $group_ndx == 0 ? 2 : 1;
            if ($group_size >= $max_group_size)
                continue;

            if ($t->id == $torrent_id)
                return true;

            $group_size++;
            $num_added++;

        }
        return false;
    }

    public function get_all_folder_items(
        MediaURL $media_url, &$plugin_cookies)
    {
        $items = array();

        $th = MainHelper::$torrent_history;
        $dl = MainHelper::$dl_state;
        $mts = MainHelper::$state->nav_mts;

        $mts->ensure_torrents_info(false);

        $chosen_only = $media_url->chosen_only == 1;
        $torrents = $chosen_only ?
            $mts->get_chosen_torrents($media_url->season, $media_url->episode) :
            $mts->get_torrents($media_url->season, $media_url->episode);

        $ndx = 0;
        foreach ($torrents as $t)
        {
            $inf = $mts->get_torrent_info($t->id);
            $good = $mts->is_good_torrent($t->id) && !$inf->invalid;

            $n = $ndx + 1;
            $caption = $n . ". " . $inf->get_caption($media_url->season);

            $item_murl = TorrentHelper::get_torrent_item_media_url(
                $media_url->season, $t->id);

            $gstat = HD::get($dl->gstat, $t->group_id);
            $status = $gstat ? HD::get($gstat, 'status', 'N') : '';
            if ($status == 'L')
                $this->set_fast_timer();

            if ($status == 'O') {
                $url = 'plugin_file://%shell_ext%/icons/sources_small.png';
            } else if (in_array($status, array('L', 'N'))) {
                $url = 'plugin_file://icons/dl_small.png';
            } else {
                $url = $good ?
                    'plugin_file://%shell_ext%/icons/movies_small.png' :
                    'plugin_file://%shell_ext%/icons/unknown_file_small.png';
            }

            $vip = array(ViewItemParams::icon_path => $url);
            $vip[ViewItemParams::item_caption_dy] = -2;
            $vip[ViewItemParams::item_caption_sel_dy] = -2;

            $eitems = array();

            $extra_text = HD::get($t, 'extra_item');
            if ($extra_text) {
                $eitems[] = array('type' => 'gap', 'width' => 20);
                $eitems[] = array('type' => 'text', 'color' => '23',
                    'size' => 'small', 'text' => $extra_text);
            }

            $tag = $th->make_torrent_tag($t->uid);
            $witem = HD::get($th->witem_by_tag, $tag);
            TorrentHistory::update_vip_eitems_by_witem($eitems, $witem);

            if ($status == 'L' && $gstat->progress >= 0)
            {
                if ($eitems)
                {
                    $gap = array_shift($eitems);
                    if ($gap['type'] == 'gap')
                        $gap['width'] = 10;
                    array_unshift($eitems, $gap);
                }

                $text = "[$gstat->progress%]";
                array_unshift($eitems, 
                    array('type' => 'text', 'color' => '34',
                        'size' => 'small', 'text' => "$text"));
                array_unshift($eitems, 
                    array('type' => 'gap', 'width' => 20));
            }

            if ($eitems) {
                $einf = array('items' => $eitems);
                $vip[ViewItemParams::item_extra_info] = json_encode($einf);
            }

            $items[] = array(
                PluginRegularFolderItem::media_url => $item_murl,
                PluginRegularFolderItem::caption => $caption,
                PluginRegularFolderItem::view_item_params => $vip,
            );

            $ndx++;
        }

        $total = count(
            $mts->get_torrents(
                $media_url->season, $media_url->episode));
        if ($chosen_only && count($torrents) < $total)
        {
            $url = 'plugin_file://%shell_ext%/icons/menu_folder_small.png';
            $vip = array(ViewItemParams::icon_path => $url);
            $items[] = array(
                PluginRegularFolderItem::media_url => 'all',
                PluginRegularFolderItem::caption => T::t('all_variants'),
                PluginRegularFolderItem::view_item_params => $vip,
            );
        }

        return $items;
    }

    ///////////////////////////////////////////////////////////////////////

    private function show_info($mts, $torrent_id, $name, $info)
    {
        if (!$info)
            return null;

        $info = preg_replace('/^[ |]+/', '', $info);

        $strs = explode("|", $info);
        if (count($strs) >= 2) {
            $n = max(1, count($strs) - 5);
            $s1 = trim(implode('|', array_slice($strs, 0, $n)));
            $s2 = trim(implode('|', array_slice($strs, $n)));
            $s2 = preg_replace('/ *\|\|* */', ' | ', $s2);

            if ($s1) {
                ControlFactory::add_label($defs,
                    '', $s1, array('max_lines' => 5));
                ControlFactory::add_vgap($defs, -10);
            }
            if ($s2) {
                ControlFactory::add_label($defs, '', $s2);
                ControlFactory::add_vgap($defs, -10);
            }
        }

        $movie = $mts->get_movie();
        $series = $movie->series_type ? 1 : 0;

        $inf = $mts->get_torrent_info($torrent_id);

        ControlFactory::add_label($defs,
            '', "Quality: $inf->quality_str ($inf->quality)");
        if ($this->debug) {
            ControlFactory::add_vgap($defs, -10);
            ControlFactory::add_label($defs,
                '', "Suffix: $inf->tail", array('max_lines' => 2));
        }
        if ($series) {
            ControlFactory::add_vgap($defs, -10);
            if ($inf->single_episode >= 0) {
                $s = $inf->seasons[0];
                $e = $inf->single_episode;
                $label = "Season $s, episode $e";
            } else {
                $end = count($inf->seasons) > 1 ? 's' : '';
                $label = "Season$end: ".implode(',',$inf->seasons);
            }
            ControlFactory::add_label($defs, '', $label);
        }
        if ($this->debug) {
            $t = $mts->get_torrent_by_id($torrent_id);

            ControlFactory::add_vgap($defs, -10);
            ControlFactory::add_label($defs,
                '', "Years: $inf->years");
            ControlFactory::add_vgap($defs, -10);
            ControlFactory::add_label($defs,
                '', "Prefix: $inf->name", array('max_lines' => 2));
            ControlFactory::add_vgap($defs, -10);
            ControlFactory::add_label($defs, '', "Flags: $inf->flags");
            ControlFactory::add_vgap($defs, -10);
            ControlFactory::add_label($defs, '', "UID: " . HD::get($t, 'uid', $torrent_id));
            ControlFactory::add_vgap($defs, -10);
            ControlFactory::add_label($defs, '', "Movie: $movie->id, $movie->title ($movie->year)");

            ControlFactory::add_vgap($defs, -10);
            ControlFactory::add_label($defs, '', "Query: $t->query");

            hd_print("DEBUG Movie: $movie->id, $movie->title ($movie->year)");
        }

        ControlFactory::add_vgap($defs, 20);
        ControlFactory::add_close_dialog_button($defs,
            T::g('button_ok'), 250);

        return ActionFactory::show_dialog(
            T::g('navigator_popup_info'), $defs, true, 0);
    }

    private function enter($movie_id, $season,
        $chosen_only, $torrent_id, $stack)
    {
        $st = MainHelper::$state;
        $mts = $st->nav_mts;
        $t = $mts->get_torrent_by_id($torrent_id);

        $child_stack = $stack .
            ($chosen_only ? 'TC:' : 'TA:') . $torrent_id . ';';

        $res = TorrentHelper::open_torrent(
            0, -1, $movie_id, $season, $torrent_id, $t->uid, $child_stack,
            $this->chosen_file_id, $this->chosen_file_pos, $this->chosen_op);

        $err_action = TorrentHelper::open_torrent_error_action($res, false);
        if ($err_action)
            return $err_action;

        $stat = MainHelper::$pb_stat;
        $stat->set_switch_reason('');

        return is_array($res) ? $res : null;
    }

    private function menu_playback_finish($user_input)
    {
        return TorrentHelper::menu_playback_finish($user_input);
    }

    private function b_green($user_input, $parent_murl)
    {
        $ss = MainHelper::$settings;

        $ss->toggle_value('sort_video_streams');

        $st = MainHelper::$state;
        $mts = $st->nav_mts;
        $mts->apply_sort_change();

        $parent_murl_str = $user_input->parent_media_url;

        $arr = array($parent_murl_str);
        $chosen_only = $parent_murl->chosen_only == 1;
        if (!$chosen_only)
        {
            $stack = strrpos($parent_murl->stack, ';', -2);
            $arr[] = TorrentsScreen::get_media_url_str(
                $parent_murl->movie_id,
                $parent_murl->season, $parent_murl->episode,
                true,
                $stack === false ? '' : $stack);
        }
        return ActionFactory::invalidate_folders($arr);
    }

    private function popup_add_dl_items(&$menu_items,
        $parent_murl, $murl)
    {
        if (!$murl)
            return;
        if (!is_dir("/tmp/run/dl_input"))
            return;
        if (!is_dir("/tmp/mnt/D/dune_downloads"))
            return;

        $t = MainHelper::$state->nav_mts->
            get_torrent_by_id($murl->torrent_id);
        $dl = MainHelper::$dl_state;
        $dl->reload_state($t->group_id);

        if ($dl->f_by_path)
        {
            $menu_items[] = ActionFactory::get_menu_item(
                T::g('navigator_popup_delete'),
                MainHelper::$entry_handler->custom_action(
                    'delete_torrent_data',
                    array('torrent_id' => $murl->torrent_id)));

            foreach ($dl->f_by_path as $path => $f)
            {
                $status = $f ? HD::get($f, 'status', 'N') : '';
                if ($status && $status != 'N')
                {
                    $menu_items[] = ActionFactory::get_menu_item(
                        T::t('popup_action_show_in_folder'),
                        MainHelper::$entry_handler->custom_action(
                            'show_in_folder',
                            array('torrent_id' => $murl->torrent_id,
                                'file_path' => $path)));
                    break;
                }
            }
        }
        else
        {
            $menu_items[] = ActionFactory::get_menu_item(
                T::t('popup_action_download'),
                $this->custom_action('enter',
                    array('chosen_op' => 'download')));
        }
    }

    private function popup_add_sort_items(&$menu_items,
        $user_input, $parent_murl)
    {
        $ss = MainHelper::$settings;
        $caption = $ss->get_value_caption('sort_video_streams');
        $action_name = T::t1('action_sort__1', $caption);
        $menu_items[] = ActionFactory::get_menu_item(
            $action_name,
            $this->custom_action('b_green'));
    }

    private function popup($user_input, $parent_murl, $murl)
    {
        $menu_items = array();
        $this->popup_add_dl_items($menu_items,
            $parent_murl, $murl);
        $this->popup_add_sort_items($menu_items,
            $user_input, $parent_murl);
        if (!$menu_items)
            return null;
        return ActionFactory::show_popup_menu($menu_items);
    }

    public function set_fast_timer()
    {
        $this->fast_timer_countdown = 10;
    }

    public function speed_up_timer($parent_murl, &$plugin_cookies)
    {
        $this->set_fast_timer();
        return ActionFactory::change_behaviour(
            $this->get_action_map($parent_murl, $plugin_cookies),
            $this->get_timer($parent_murl));
    }

    public function handle_user_input(&$user_input, &$plugin_cookies)
    {
        if (HD::get($user_input, 'control_id') == 'timer') {
            hd_print('torrents D: timer');
        } else {
            hd_print('Torrents screen: handle_user_input:');
            foreach ($user_input as $key => $value)
                hd_print("  $key => $value");
        }

        $eh = MainHelper::$entry_handler;

        $murl_str = HD::get($user_input, 'chosen_media_url',
            HD::get($user_input, 'selected_media_url'));

        $murl = $murl_str ? json_decode($murl_str) : null;

        $parent_murl = MediaURL::decode($user_input->parent_media_url);
        $st = MainHelper::$state;
        $mts = $st->nav_mts;

        if ($user_input->control_id == 'info') {
            if (!$murl)
                return null;
            $t = $mts->get_torrent_by_id($murl->torrent_id);
            return $this->show_info($mts,
                $t->id, $t->name, $t->info);
        }

        if ($user_input->control_id == 'b_green')
        {
            return $this->b_green($user_input, $parent_murl);
        }

        if ($user_input->control_id == 'popup') {
            return $this->popup($user_input, $parent_murl, $murl);
        }

        if ($user_input->control_id == 'enter')
        {
            if ($user_input->selected_media_url == 'all')
            {
                $st->enter_all_torrents();

                $stack = $parent_murl->stack . "TC:all;";
                $child_murl = TorrentsScreen::get_media_url_str(
                    $parent_murl->movie_id,
                    $parent_murl->season, $parent_murl->episode,
                    false, $stack);
                return ActionFactory::open_folder($child_murl);
            }

            if (!$murl)
                return null;

            $this->chosen_file_id = HD::get($user_input, 'chosen_file_id');
            $this->chosen_file_pos = HD::get($user_input, 'chosen_file_pos', -1);
            $this->chosen_op = HD::get($user_input, 'chosen_op', 'enter');

            if ($this->chosen_op != 'download' &&
                $parent_murl->chosen_only &&
                isset($user_input->chosen_media_url) &&
                !$this->is_collapsed_torrent_id($mts,
                    $parent_murl->season, $parent_murl->episode, $murl->torrent_id))
            {
                $sel_id = $user_input->chosen_media_url;
                $params = array('chosen_media_url' => $sel_id);
                if (isset($this->chosen_file_id))
                    $params['chosen_file_id'] = $this->chosen_file_id;
                $params['chosen_file_pos'] = $this->chosen_file_pos;
                $params['chosen_op'] = $this->chosen_op;
                $post_action = $this->custom_action('enter', $params);

                $st->enter_all_torrents();

                $stack = $parent_murl->stack . "TC:all;";
                return ActionFactory::open_folder_ext(
                    TorrentsScreen::get_media_url_str(
                        $parent_murl->movie_id,
                        $parent_murl->season, $parent_murl->episode,
                        false, $stack),
                    T::t('all_variants'), null, $sel_id, $post_action);
            }

            if (isset($user_input->chosen_media_url) &&
                $this->chosen_op == 'enter' &&
                !$this->chosen_file_id)
            {
                return null;
            }

            if (!HD::get($user_input, 'torrserver_checked')) {
                $cb = ActionFactory::action_from_user_input(
                    $user_input,
                    array('control_id' => 'enter',
                        'torrserver_checked' => 1));
                return $eh->ensure_own_torrserver($user_input, $cb);
            }

            return $this->enter($parent_murl->movie_id,
                $parent_murl->season, $parent_murl->chosen_only,
                $murl->torrent_id, $parent_murl->stack);
        }

        if ($user_input->control_id == 'menu_playback_finish')
            return $this->menu_playback_finish($user_input);

        if ($user_input->control_id == 'timer')
        {
            $dl = MainHelper::$dl_state;
            $changed = $dl->reload_gstat();
            $a = ActionFactory::change_behaviour(
                $this->get_action_map($parent_murl, $plugin_cookies),
                $this->get_timer($parent_murl));
            if ($this->fast_timer_countdown > 0)
                $this->fast_timer_countdown--;

            if ($changed)
            {
                $a = ActionFactory::invalidate_folders(
                    array($user_input->parent_media_url), $a);
            }

            return $a;
        }

        return null;
    }
}
?>
