<?php

class UrlHelper
{
    private static function parse_stack($stack) {
        $arr = array();
        $pos = 0;
        for (;;)
        {
            $pos2 = strpos($stack, ';', $pos);
            if ($pos2 === false)
                break;

            $item = substr($stack, $pos, $pos2 - $pos);
            list($type,$id) = explode(':', $item);
            $arr[] = (object) array('type' => $type, 'id' => $id);

            $pos = $pos2 + 1;
        }
        return $arr;
    }

    private static function stack_item_to_str($item) {
        return "$item->type:$item->id;";
    }

    public static function get_urls_stack($user_input)
    {
        $urls = array();

        $parent_murl_str = HD::get($user_input, 'parent_media_url');
        if (!$parent_murl_str)
            return $urls;
        $parent_murl = MediaURL::decode($parent_murl_str);
        $movie_id = HD::get($parent_murl, 'movie_id');
        if (!$movie_id)
            return $urls;

        $stack = HD::get($parent_murl, 'stack', '');
        $sarr = self::parse_stack($stack);

        $stack2 = '';
        $s = -1;
        $e = -1;
        $all = false;
        $torrent_id = -1;

        foreach ($sarr as $item)
        {
            if ($item->type == 'S') {
                $s = $item->id;
                $urls[] = "S$item->id"; // TODO: revise
            } else if ($item->type == 'E') {
                $e = $item->id;
                $urls[] = "E$item->id"; // TODO: revise
            } else if ($item->type == 'S2') {
                $s = $item->id;
                $urls[] = SeasonsScreen::get_media_url_str($movie_id);
            } else if ($item->type == 'TC') {
                if ($item->id == 'all')
                    $all = true;
                else
                    $torrent_id = $item->id;
                $urls[] = TorrentsScreen::get_media_url_str($movie_id,
                    $s, $e, true, $stack2);
            } else if ($item->type == 'TA') {
                $torrent_id = $item->id;
                $urls[] = TorrentsScreen::get_media_url_str($movie_id,
                    $s, $e, false, $stack2);
            } else if ($item->type == 'TF') {
                $urls[] = TorrentFilesScreen::get_media_url_str($movie_id,
                    $torrent_id, $stack2);
            }

            $stack2 .= self::stack_item_to_str($item);
        }

        $urls[] = $parent_murl_str;

        return $urls;
    }

    public static function get_season_caption($s)
    {
        if ($s <= 0)
            return T::t('item_others');
        return T::g1('shell_ext_plugin_item_season__1', $s);
    }

    public static function replace_path_after_playback($user_input)
    {
        $from_osd_screen = HD::get($user_input, 'from_osd_screen');
        if ($from_osd_screen != 'yes') {
            hd_print("DEBUG replace_path: not osd screen");
            return null;
        }

        $st = MainHelper::$state;
        $play_mts = $st->play_mts;
        $play_tfs = $st->play_tfs;
        $playstart_tfs = $st->playstart_tfs;
        if (!$play_mts || !$play_tfs || !$playstart_tfs) {
            hd_print("DEBUG replace_path: no play mts/tfs (illegal state?)");
            return null;
        }

/*
        if ($play_tfs->get_torrent_id() == $playstart_tfs->get_torrent_id()) {
            hd_print("DEBUG replace_path: play torrent not changed");
            return null;
        }
*/

        $parent_murl = MediaURL::decode($user_input->parent_media_url);
        $screen_id = $parent_murl->screen_id;
        $is_files_screen = $screen_id == TorrentFilesScreen::ID;
        if (!$is_files_screen) {
            // TODO: implement for torrents screen
            hd_print("DEBUG replace_path: not needed for screen_id: $screen_id");
            return null;
        }

        if ($parent_murl->torrent_id != $playstart_tfs->get_torrent_id()) {
            hd_print("DEBUG replace_path: nav stack changed");
            return null;
        }

        if ($st->nav_mts->get_movie_id() != $play_mts->get_movie_id()) {
            // Should not happen, but for the case..
            hd_print("DEBUG replace_path: movie changed unexpectedly");
            return null;
        }

        $torrent_id = $play_tfs->get_torrent_id();
        $m = $play_mts->get_movie();
        $s = $play_mts->get_season();
        $e = $play_mts->get_episode();
        $t = $play_mts->get_torrent_by_id($torrent_id);
        if (!$t) {
            hd_print("DEBUG replace_path: play torrent not found (illegal state?)");
            return null;
        }
        $inf = $play_mts->get_torrent_info($torrent_id);
        if (!$inf) {
            hd_print("DEBUG replace_path: play torrent info missing (illegal state?)");
            return null;
        }

        if ($play_mts->get_autostart()) {
            hd_print("DEBUG replace_path: not needed in autostart context");
            return null;
        }

        $playback_url = HD::get($user_input, 'url');
        if (!$playback_url) {
            hd_print("DEBUG replace_path: playback url missing");
            return null;
        }
        $ser = $play_tfs->get_ser_by_playback_url($playback_url);
        if (!$ser) {
            hd_print("DEBUG replace_path: series not found by playback url");
            return null;
        }

        $stack = $parent_murl->stack;
        $sarr = self::parse_stack($stack);
        self::add_top_stack_item($sarr, $parent_murl,
            HD::get($user_input, 'selected_media_url'));

        $sarr2 = array();

        if ($m->series_type) {
            $sarr2[] = (object) array('type' => 'S2', 'id' => $s);
        }

        $chosen = $play_mts->is_chosen_torrent_id($torrent_id);
        if ($chosen) {
            $sarr2[] = (object) array('type' => 'TC', 'id' => $torrent_id);
        } else {
            $sarr2[] = (object) array('type' => 'TC', 'id' => 'all');
            $sarr2[] = (object) array('type' => 'TA', 'id' => $torrent_id);
        }

        $sarr2[] = (object) array('type' => 'TF', 'id' => $ser['id']);

        list($erase_count, $elements, $sel_id) =
            self::stack_to_replace_path_action(
                $m->id, $s, $e, $torrent_id, $inf, $sarr, $sarr2);

#$n = count($elements);
#hd_print("DEBUG replace_path: $n elements: ".print_r($elements,1));
#hd_print("DEBUG replace_path sel_id: $sel_id");

        if (!$erase_count && !$elements) {
            hd_print("DEBUG replace_path: no changes (2)");
            return null;
        }

        $st->enter_season_and_episode($s, $e);

        $st->enter_torrent($st->play_tfs);

        return ActionFactory::replace_path(
            $erase_count, $elements, $sel_id);
    }

    private static function stack_to_replace_path_action(
        $movie_id, $s, $e, $torrent_id, $inf, $sarr1, $sarr2)
    {
        $empty_res = array(0, array(), null);

        $sz1 = count($sarr1);
        $sz2 = count($sarr2);
        if ($sz1 == 0 && $sz2 == 0)
            return $empty_res;

        for ($ndx = 0;; $ndx++) {
            $si1 = HD::get($sarr1, $ndx);
            $si2 = HD::get($sarr2, $ndx);
            if (!$si1 || !$si2 || $si1->type != $si2->type)
                break;
            if ($si1->id != $si2->id) {
                $ndx++;
                break;
            }
        }

hd_print("sz1 $sz1, sz2 $sz1, ndx $ndx");

        $erase_count = $sz1 - $ndx;
        $add_count = $sz2 - $ndx;
hd_print("DEBUG replace_path: erase $erase_count, add $add_count");

        if ($erase_count == 0 && $add_count == 0) {
            $id1 = $sarr1[$sz1 - 1]->id;
            $id2 = $sarr2[$sz2 - 1]->id;
            if ($id1 == $id2) {
                hd_print("DEBUG replace_path: not changed");
                return $empty_res;
            }
        }

        $elements = array();

        $id = self::get_path_item_id(null);
        $caption = self::get_path_item_caption(null);
        $stack2 = '';
        for ($i = 0; $i < $sz2; $i++) {
            $item = $sarr2[$i];

            if ($i >= $ndx) {
                $murl_str = self::get_path_item_media_url_str($item,
                    $movie_id, $s, $e, $torrent_id, $stack2);
                ActionFactory::add_path_element($elements,
                    $murl_str, $caption, $id);
            }

            $id = self::get_path_item_id($item, $s);
            $caption = self::get_path_item_caption($item, $inf, $s);
            $stack2 .= self::stack_item_to_str($item);
        }

        return array($erase_count, $elements, $id);
    }

    public static function get_path_item_caption($item, $inf=null, $s=-1)
    {
        if (!$item)
            return T::t('title_plugin');

        if ($item->type == 'S2')
            return UrlHelper::get_season_caption($s);

        if ($item->type == 'TC' && $item->id == 'all')
            return T::t('all_variants');
        if ($item->type == 'TC' || $item->type == 'TA')
            return $inf->get_caption($s);

        return '';
    }

    public static function get_path_item_id($item, $s=-1)
    {
        if (!$item)
            return '';

        if ($item->type == 'S2')
            return SeasonsScreen::get_item_media_url($item->id);

        if ($item->type == 'TC' && $item->id == 'all')
            return 'all';
        if ($item->type == 'TC' || $item->type == 'TA')
            return TorrentHelper::get_torrent_item_media_url($s, $item->id);

        if ($item->type == 'TF')
            return TorrentFilesScreen::get_item_media_url($item->id);

        return '';
    }

    public static function get_path_item_media_url_str($item,
        $movie_id, $s, $e, $torrent_id, $stack)
    {
        if ($item->type == 'S2')
            return SeasonsScreen::get_media_url_str($movie_id);

        if ($item->type == 'TC' && $item->id == 'all') {
            return TorrentsScreen::get_media_url_str(
                $movie_id, $s, $e, true, $stack);
        }

        if ($item->type == 'TC' || $item->type == 'TA') {
            return TorrentsScreen::get_media_url_str(
                $movie_id, $s, $e, false, $stack);
        }

        if ($item->type == 'TF') {
            return TorrentFilesScreen::get_media_url_str(
                $movie_id, $torrent_id, $stack);
        }

        return '';
    }

    public static function add_top_stack_item(&$sarr, $parent_murl, $sel_str)
    {
        if (!$sel_str)
            return;

        if ($parent_murl->screen_id == TorrentFilesScreen::ID) {
            $sel = json_decode($sel_str);
            $sarr[] = (object) array('type' => 'TF', 'id' => $sel->id);
        } else if ($parent_murl->screen_id == TorrentsScreen::ID) {
            $sel = $sel_str == 'all' ? null : json_decode($sel_str);
            $id = $sel ? $sel->id : 'all';
            $type = $parent_murl->chosen_only ? 'TC' : 'TA';
            $sarr[] = (object) array('type' => $type, 'id' => $id);
        }
    }
}

?>
